from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from BTrees.OOBTree import OOBTree
from castle.cms import cache
from castle.cms.constants import ALL_SUBSCRIBERS
from castle.cms.constants import ALL_USERS
from castle.cms.constants import MAX_PASTE_ITEMS
from castle.cms.interfaces import IHasDefaultImage
from castle.cms.interfaces import IReferenceNamedImage
from collective.elasticsearch.es import ElasticSearchCatalog
from collective.elasticsearch.hook import index_batch
from collective.elasticsearch.interfaces import IElasticSettings
from DateTime import DateTime
from datetime import datetime
from elasticsearch import Elasticsearch
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from hashlib import sha256 as sha
from html2text import html2text
from lxml.html import fromstring
from lxml.html import tostring
from lxml.html.clean import Cleaner
from OFS.CopySupport import _cb_decode
from OFS.CopySupport import CopyError
from OFS.CopySupport import eInvalid
from persistent.dict import PersistentDict
from persistent.list import PersistentList
from plone import api
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.app.querystring import queryparser
from plone.app.querystring.interfaces import IParsedQueryIndexModifier
from plone.app.uuid.utils import uuidToCatalogBrain
from plone.dexterity.interfaces import IDexterityContent
from plone.dexterity.interfaces import IDexterityEditForm
from plone.registry.interfaces import IRegistry
from plone.subrequest import subrequest
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from urllib import unquote
from ZODB.POSException import ConflictError
from ZODB.POSException import POSKeyError
from zope.component import getUtilitiesFor
from zope.component import getUtility
from zope.component import queryUtility
from zope.globalrequest import getRequest
from zope.security.interfaces import IPermission

import hashlib
import json
import logging
import os
import random
import re
import requests
import time
import transaction
import types


try:
    random = random.SystemRandom()
    using_sysrandom = True
except NotImplementedError:

    using_sysrandom = False

SECRET = random.randint(0, 1000000)
logger = logging.getLogger('castle.cms')
ANONYMOUS_USER = "Anonymous User"


_truncate_cleaner = Cleaner(
    scripts=True, javascript=True, comments=True, style=True, links=True, meta=True,
    page_structure=True, embedded=True, frames=True, forms=True, annoying_tags=True,
    remove_tags=('div',), kill_tags=('img', 'hr'), remove_unknown_tags=True)


def truncate_text(text, max_words=30, more_link=None, clean=False):
    """
    adapted from Django
    """

    if not isinstance(text, basestring):
        return ''

    if clean:
        try:
            if not isinstance(text, unicode):
                text = text.decode('utf8')
            xml = fromstring(text)
            _truncate_cleaner(xml)
            # also remove empty tags...
            for p in xml.xpath("//p"):
                if len(p):
                    continue
                t = p.text
                if not (t and t.replace('&nbsp;', '').strip()):
                    p.getparent().remove(p)
            text = tostring(xml)
        except:
            pass
    length = int(max_words)
    if length <= 0:
        return u''
    html4_singlets = ('br', 'col', 'link', 'base',
                      'img', 'param', 'area', 'hr', 'input')
    # Set up regular expressions
    re_words = re.compile(r'&.*?;|<.*?>|(\w[\w-]*)', re.U)
    re_tag = re.compile(r'<(/)?([^ ]+?)(?: (/)| .*?)?>')
    # Count non-HTML words and keep note of open tags
    pos = 0
    end_text_pos = 0
    words = 0
    open_tags = []
    while words <= length:
        m = re_words.search(text, pos)
        if not m:
            # Checked through whole string
            break
        pos = m.end(0)
        if m.group(1):
            # It's an actual non-HTML word
            words += 1
            if words == length:
                end_text_pos = pos
            continue
        # Check for tag
        tag = re_tag.match(m.group(0))
        if not tag or end_text_pos:
            # Don't worry about non tags or tags after our truncate point
            continue
        closing_tag, tagname, self_closing = tag.groups()
        tagname = tagname.lower()  # Element names are always case-insensitive
        if self_closing or tagname in html4_singlets:
            pass
        elif closing_tag:
            # Check for match in open tags list
            try:
                i = open_tags.index(tagname)
            except ValueError:
                pass
            else:
                # SGML: An end tag closes, back to the matching start tag,
                # all unclosed intervening start tags with omitted end tags
                open_tags = open_tags[i + 1:]
        else:
            # Add it to the start of the open tags list
            open_tags.insert(0, tagname)

    if words <= length:
        # Don't try to close tags if we don't need to truncate
        return text
    out = text[:end_text_pos]
    out += '&hellip;'
    if more_link:
        out += ' <a href="%s">more &#x2192;</a>' % more_link
    # Close any tags still open
    for tag in open_tags:
        out += '</%s>' % tag
    # Return string
    return out
truncateText = truncate_text


def get_random_string(length=12,
                      allowed_chars='abcdefghijklmnopqrstuvwxyz'
                                    'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'):
    """
    Returns a securely generated random string.

    The default length of 12 with the a-z, A-Z, 0-9 character set returns
    a 71-bit value. log_2((26+26+10)^12) =~ 71 bits
    """
    if not using_sysrandom:
        # This is ugly, and a hack, but it makes things better than
        # the alternative of predictability. This re-seeds the PRNG
        # using a value that is hard for an attacker to predict, every
        # time a random string is required. This may change the
        # properties of the chosen random sequence slightly, but this
        # is better than absolute predictability.
        random.seed(
            sha(
                "%s%s%s" % (
                    random.getstate(),
                    time.time(),
                    SECRET)
                ).digest())  # noqa
    return ''.join([random.choice(allowed_chars) for i in range(length)])


def make_random_key(length=150, prefix=''):
    if prefix:
        prefix = str(prefix) + '-'
    hashed = '%s%s' % (
        hashlib.sha1(str(get_random_string(length)).encode('utf-8')).hexdigest()[:5],  # noqa
        str(datetime.now().microsecond)
    )
    return prefix + hashlib.sha1(hashed.encode('utf-8')).hexdigest()[:length]


def get_paste_data(req):
    try:
        op, mdatas = _cb_decode(req['__cp'])
    except:
        raise CopyError(eInvalid)

    try:
        if mdatas[0][0].startswith('cache:'):
            cache_key = mdatas[0][0].replace('cache:', '')
            mdatas = cache.get(cache_key)
    except IndexError:
        pass

    paths = []
    for mdata in mdatas:
        paths.append('/'.join(mdata))

    catalog = api.portal.get_tool('portal_catalog')
    count = len(catalog(path={'query': paths, 'depth': -1}))

    return {
        'paths': paths,
        'op': op,
        'mdatas': mdatas,
        'count': count
    }


def is_max_paste_items(paste_data):
    return paste_data['count'] > MAX_PASTE_ITEMS


def recursive_create_path(site, path):
    if path == '/':
        folder = site
    else:
        path = path.lstrip('/')
        folder = site.restrictedTraverse(path, None)
        if folder is None:
            # Need to create folders up to where we want content
            # we'll walk it up create folders as needed
            folder = site
            for part in path.split('/'):
                try:
                    folder = folder[part]
                except KeyError:
                    try:
                        folder = api.content.create(
                            type='Folder',
                            id=part,
                            title=part.capitalize().replace('-', ' '),
                            container=folder)
                    except:
                        raise
    return folder


def retriable(count=3, sync=False, reraise=True, on_retry_exhausted=None):

    def decorator(func):
        def wrapped(*args, **kwargs):
            retried = 0
            if sync:
                try:
                    api.portal.get()._p_jar.sync()
                except:
                    pass
            while retried < count:
                try:
                    return func(*args, **kwargs)
                except ConflictError:
                    retried += 1
                    try:
                        api.portal.get()._p_jar.sync()
                    except:
                        if retried >= count:
                            if on_retry_exhausted is not None:
                                on_retry_exhausted(*args, **kwargs)
                            if reraise:
                                raise
        return wrapped
    return decorator


def clear_object_cache(ob):
    ob._p_jar.invalidateCache()
    transaction.begin()
    ob._p_jar.sync()


def index_in_es(obj):
    catalog = api.portal.get_tool('portal_catalog')
    es = ElasticSearchCatalog(catalog)
    if es.enabled:
        index_batch([], {IUUID(obj): obj}, [], es)


def verify_recaptcha(req=None):
    if req is None:
        req = getRequest()

    registry = getUtility(IRegistry)
    key = registry.get('castle.recaptcha_private_key', '')
    if not key:
        # do not bother verifying if no key is defined
        return True
    code = req.form.get('g-recaptcha-response', '')
    remote_addr = req.get(
        'HTTP_X_FORWARDED_FOR',
        ''
    ).split(',')[0]
    if not remote_addr:
        remote_addr = req.get('REMOTE_ADDR')

    resp = requests.post(
        'https://www.google.com/recaptcha/api/siteverify',
        data=dict(
            secret=key,
            response=code,
            remoteip=remote_addr
        )
    )
    try:
        return resp.json()['success']
    except:
        return False


def get_email_from_address():
    registry = getUtility(IRegistry)
    mail_settings = registry.forInterface(IMailSchema, prefix='plone')
    return mail_settings.email_from_address


def send_email(recipients=None, subject=None, html='', text='', sender=None):
    if isinstance(recipients, basestring):
        recipients = [recipients]

    cleaned_recipients = []
    for recipient in recipients:
        if recipients == ALL_USERS:
            for user in api.user.get_users():
                email = user.getProperty('email')
                if email:
                    cleaned_recipients.append(email)
        elif recipients == ALL_SUBSCRIBERS:
            from castle.cms import subscribe
            cleaned_recipients.extend(subscribe.get_email_addresses())
        else:
            cleaned_recipients.append(recipient)

    if sender is None:
        sender = get_email_from_address()

    if not text and html:
        try:
            text = html2text(html)
        except:
            pass

    for recipient in cleaned_recipients:
        msg = MIMEMultipart('alternative')
        msg['Subject'] = subject
        msg['From'] = sender
        msg['To'] = recipient
        if text:
            part = MIMEText(text, 'plain')
            msg.attach(part)
        if html:
            part = MIMEText(html, 'html')
            msg.attach(part)

        mailhost = api.portal.get_tool('MailHost')
        mailhost.send(msg.as_string())


def inline_images_in_dom(dom, portal=None, site_url=None):
    if portal is None:
        portal = api.portal.get()
    if site_url is None:
        site_url = portal.absolute_url()
    for img in dom.cssselect('img'):
        src = img.attrib.get('src', '')
        if src.startswith(site_url):
            ctype, data = get_data_from_url(src, portal, site_url)
            if not ctype or not data or 'image' not in ctype:
                img.attrib['src'] = ''
                continue
            data = data.encode("base64").replace("\n", "")
            data_uri = 'data:{0};base64,{1}'.format(ctype, data)
            img.attrib['src'] = data_uri


def ESConnectionFactoryFactory(registry=None):
    if registry is None:
        registry = getUtility(IRegistry)
    settings = registry.forInterface(IElasticSettings)
    hosts = settings.hosts
    opts = dict(
        timeout=getattr(settings, 'timeout', 0.5),
        sniff_on_start=getattr(settings, 'sniff_on_start', False),
        sniff_on_connection_fail=getattr(settings, 'sniff_on_connection_fail', False),
        sniffer_timeout=getattr(settings, 'sniffer_timeout', 0.1),
        retry_on_timeout=getattr(settings, 'retry_on_timeout', False)
    )

    def factory():
        return Elasticsearch(hosts, **opts)
    return factory


def add_indexes(indexes):
    """
    indexes should be a tuple of (name of the index, index type)
    """
    catalog = api.portal.get_tool('portal_catalog')
    for name, _type in indexes.items():
        if name not in catalog.indexes():
            if type(_type) == dict:
                real_type = _type['type']
                del _type['type']
                catalog.addIndex(name, real_type, **_type)
            else:
                catalog.addIndex(name, _type)


def delete_indexes(indexes):
    catalog = api.portal.get_tool('portal_catalog')
    for name in indexes:
        if name in catalog.indexes():
            catalog.delIndex(name)


def add_metadata(metadata):
    catalog = api.portal.get_tool('portal_catalog')
    _catalog = catalog._catalog
    for name in metadata:
        if name not in catalog.schema():
            # override how this works normally to not cause a reindex
            schema = _catalog.schema
            names = list(_catalog.names)
            values = schema.values()
            if values:
                schema[name] = max(values) + 1
            else:
                schema[name] = 0
            names.append(name)

            _catalog.names = tuple(names)
            _catalog.schema = schema


def delete_metadata(metadata):
    catalog = api.portal.get_tool('portal_catalog')
    for name in metadata:
        if name in catalog.schema():
            catalog.delColumn(name)


def is_mosaic_edit_form(request):
    try:
        if (ILayoutAware.providedBy(request.PUBLISHED.context) and
                IDexterityEditForm.providedBy(request.PUBLISHED) and
                request.PUBLISHED.context.getLayout() == 'layout_view' and
                request.PUBLISHED.__name__ == 'edit'):
            return True
    except:
        pass
    return False


def get_ip(req):
    ip = req.get('HTTP_CF_CONNECTING_IP')
    if not ip:
        ip = req.get('HTTP_X_FORWARDED_FOR')
    if not ip:
        ip = req.get('HTTP_X_REAL_IP')
    if not ip:
        ip = req.get('REMOTE_ADDR')
    return ip


def get_backend_url():
    registry = getUtility(IRegistry)
    backend_url = registry.get('plone.backend_url', None)
    if not backend_url:
        backend_url = api.portal.get().absolute_url()
    return backend_url


def parse_query_from_data(data, context=None):
    if context is None:
        context = api.portal.get()
    parsed = queryparser.parseFormquery(context,
                                        data.get('query', {}) or {})

    index_modifiers = getUtilitiesFor(IParsedQueryIndexModifier)
    for name, modifier in index_modifiers:
        if name in parsed:
            new_name, query = modifier(parsed[name])
            parsed[name] = query
            # if a new index name has been returned, we need to replace
            # the native ones
            if name != new_name:
                del parsed[name]
                parsed[new_name] = query

    if data.get('sort_on'):
        parsed['sort_on'] = data['sort_on']
    if data.get('sort_reversed', False):
        parsed['sort_order'] = 'reverse'
    return parsed


def normalize_url(url):
    url = url.split('#')[0]
    if url.startswith('data:'):
        return None
    return url


def strings_differ(string1, string2):
    """Check whether two strings differ while avoiding timing attacks.

    This function returns True if the given strings differ and False
    if they are equal.  It's careful not to leak information about *where*
    they differ as a result of its running time, which can be very important
    to avoid certain timing-related crypto attacks:

        http://seb.dbzteam.org/crypto/python-oauth-timing-hmac.pdf

    """
    if len(string1) != len(string2):
        return True

    invalid_bits = 0
    for a, b in zip(string1, string2):
        invalid_bits += a != b

    return invalid_bits != 0


def get_public_url():
    public_url = api.portal.get_registry_record('plone.public_url')
    if not public_url:
        public_url = api.portal.get().absolute_url()
    return public_url


def get_object_version(obj, version):
    context = aq_inner(obj)
    if version == "current":
        return context
    else:
        repo_tool = api.portal.get_tool('portal_repository')
        return repo_tool.retrieve(context, int(version)).object


def _customhandler(obj):
    if type(obj) == DateTime:
        return obj.ISO8601()
    if type(obj) == PersistentList:
        return list(obj)
    if type(obj) == PersistentDict:
        return dict(obj)
    if type(obj) == OOBTree:
        return dict(obj)
    raise TypeError(
        "Unserializable object {} of type {}".format(obj, type(obj)))


def json_dumps(data):
    return json.dumps(data, default=_customhandler)


def get_path(obj, site=None):
    """
    Get the full path FROM the site of an object
    path object: /Castle/foo/bar -> /foo/bar
    """
    if site is None:
        site = api.portal.get()
    site_path = site.getPhysicalPath()
    obj_path = obj.getPhysicalPath()
    path = '/' + '/'.join(obj_path[len(site_path):])
    return path


def get_data_from_url(url, portal=None, site_url=None):
    if portal is None:
        portal = api.portal.get()
    if site_url is None:
        site_url = portal.absolute_url()
    data = None
    path = url.replace(site_url, '').strip('/')
    ct = ''
    if '/@@images/' in path:
        try:
            path, _, im_info = path.partition('/@@images/')
            parts = im_info.split('/')
            if len(parts) == 2:
                fieldname = parts[0]
                size = parts[1]
                images = portal.restrictedTraverse(
                    str(path + '/@@images'), None)
                if images is not None:
                    im = images.traverse(fieldname, [size])
                    try:
                        data = im.scale.data.data
                        ct = im.scale.data.contentType
                    except AttributeError:
                        pass
            else:
                # grab full size
                ob = portal.restrictedTraverse(str(path), None)
                data = ob.image.data
                ct = ob.image.contentType
        except:
            logger.error('Could not traverse image ' + url, exc_info=True)

    if data is None:
        ob = portal.restrictedTraverse(str(path), None)
        file_path = getattr(ob, 'path', None)
        if file_path is None:
            try:
                file_path = ob.context.path
            except:
                pass
        if file_path and os.path.exists(file_path):
            fi = open(file_path)
            data = fi.read()
            fi.close()
            ct = 'image/' + file_path.split('.')[-1].lower()
        else:
            resp = subrequest(unquote(url))
            if resp.status != 200:
                return None, None
            try:
                ct, encoding = resp.getHeader(
                    'content-type').split('charset=')
                ct = ct.split(';')[0]
                # pisa only likes ascii css
                data = resp.getBody().decode(encoding)
            except ValueError:
                ct = resp.getHeader('content-type').split(';')[0]
                data = resp.getBody()

    return ct, data


def get_image_info(brain):
    image_info = None
    if IContentListingObject.providedBy(brain):
        brain = brain._brain
    if IDexterityContent.providedBy(brain):
        obj = brain
        try:
            image = obj.image
            if IReferenceNamedImage.providedBy(image):
                uid = image.reference
                brain = uuidToCatalogBrain(uid)
                if brain:
                    return get_image_info(brain)
                else:
                    return
            width, height = image.getImageSize()
            image_info = {
                'width': width,
                'height': height
            }
            try:
                image_info['focal_point'] = image.focal_point
            except:
                try:
                    image_info['focal_point'] = obj._image_focal_point
                except:
                    image_info['focal_point'] = [width / 2, height / 2]
        except AttributeError:
            pass
    else:
        try:
            image_info = brain.image_info
            if 'reference' in image_info:
                uid = image_info['reference']
                brain = uuidToCatalogBrain(uid)
                if brain:
                    return get_image_info(brain)
        except:
            pass

    return image_info


def get_folder_contents(folder, **query):
    query.update({
        'sort_on': 'getObjPositionInParent',
        'path': {
            'query': '/'.join(folder.getPhysicalPath()),
            'depth': 1
        },
        'show_inactive': False
    })
    catalog = api.portal.get_tool('portal_catalog')
    return catalog(**query)


def site_has_icon():
    key = '{}.has_site_icon'.format(''.join(api.portal.get().getPhysicalPath()))
    try:
        has_icon = cache.ram.get(key)
    except KeyError:
        registry = getUtility(IRegistry)
        has_icon = False
        try:
            has_icon = bool(registry['plone.site_icon'])
        except:
            pass
        cache.ram.set(key, has_icon)
    return has_icon


def get_context_from_request(request):
    published = request.get('PUBLISHED')
    if isinstance(published, types.MethodType):
        return published.im_self
    return aq_parent(published)


def get_managers():
    managers = []
    for admin_user in api.user.get_users():
        user_roles = api.user.get_roles(user=admin_user)
        admin_email = admin_user.getProperty('email')
        if (('Manager' not in user_roles and
                'Site Administrator' not in user_roles) or
                not admin_email):
            continue
        managers.append(admin_user)
    return managers


def has_image(obj):
    try:
        # check if brain has the data
        return obj.hasImage
    except:
        if getattr(obj, 'getObject', False):
            obj = obj.getObject()
        if IHasDefaultImage.providedBy(obj):
            return True
        try:
            return getattr(aq_base(obj), 'image', None)
        except POSKeyError:
            return False


def get_permission_title(permission):
    add_perm_ob = queryUtility(IPermission, name=permission)
    if add_perm_ob:
        permission = add_perm_ob.title
    return permission



from plone.app.dexterity.behaviors.metadata import ICategorization
from plone import api
from Products.Five import BrowserView
import json


class TagsView(BrowserView):
    label = 'Tag Manager'

    batch_size = 50
    remove_size = 25

    def __call__(self):
        action = self.request.form.get('action')
        if action:
            self.request.response.setHeader('Content-type', 'application/json')
            if action == 'get':
                data = self.get()
            elif action == 'remove':
                data = self.remove()
            elif action == 'rename':
                data = self.rename()
            return json.dumps(data)
        return super(TagsView, self).__call__()

    def rename(self):
        tag = self.request.form.get('tag')
        new_tag = self.request.form.get('new_tag')
        catalog = api.portal.get_tool('portal_catalog')
        for brain in catalog(Subject=tag)[:self.remove_size]:
            obj = brain.getObject()
            bdata = ICategorization(obj, None)
            if bdata:
                if tag in bdata.subjects:
                    tags = list(bdata.subjects)
                    tags.remove(tag)
                    if new_tag not in tags:
                        tags.append(new_tag)
                    bdata.subjects = tuple(tags)
                    obj.reindexObject(idxs=['Subject'])
        try:
            total = len(catalog._catalog.getIndex('Subject')._index[tag])
        except:
            total = 0
        return {
            'success': True,
            'tag': tag,
            'total': total
        }

    def remove(self):
        tag = self.request.form.get('tag')
        catalog = api.portal.get_tool('portal_catalog')
        for brain in catalog(Subject=tag)[:self.remove_size]:
            obj = brain.getObject()
            bdata = ICategorization(obj, None)
            if bdata:
                if tag in bdata.subjects:
                    tags = list(bdata.subjects)
                    tags.remove(tag)
                    bdata.subjects = tuple(tags)
                    obj.reindexObject(idxs=['Subject'])
        try:
            total = len(catalog._catalog.getIndex('Subject')._index[tag])
        except:
            total = 0
        return {
            'success': True,
            'tag': tag,
            'total': total
        }

    def get(self):
        cat = api.portal.get_tool('portal_catalog')
        index = cat._catalog.getIndex('Subject')
        tags = []
        for name in index._index.keys():
            try:
                number_of_entries = len(index._index[name])
            except TypeError:
                continue
            tags.append({
                'name': name,
                'count': number_of_entries
            })

        sorted_tags = list(reversed(sorted(tags, key=lambda tag: tag['count'])))

        try:
            page = int(self.request.form.get('page'))
        except:
            page = 1

        start = (page - 1) * self.batch_size
        end = start + self.batch_size
        return {
            'success': True,
            'tags': sorted_tags[start:end],
            'total': len(sorted_tags),
            'page': page,
            'batch_size': self.batch_size
        }



# flake8: noqa

from .authentication import IAuthenticator
from .archive import IArchiveContentTransformer
from .content import IAudio
from .content import ICastleApplication
from .content import IDashboard
from .content import IHasDefaultImage
from .content import IMedia
from .content import IReferenceNamedImage
from .content import ITrashed
from .content import IVideo
from .controlpanel import IAnnoucementData
from .controlpanel import IAPISettings
from .controlpanel import IArchivalSettings
from .controlpanel import IBusinessData
from .controlpanel import ICastleSettings
from .controlpanel import ICrawlerConfiguration
from .controlpanel import ISecuritySchema
from .controlpanel import ISiteConfiguration
from .controlpanel import ISiteSchema
from .controlpanel import ISocialMediaSchema
from .layers import ICastleLayer
from .layers import IVersionViewLayer
from .metadata import ILDData
from .tiles import IFieldTileRenderer
from .tiles import IGlobalTile
from .tiles import IMetaTile
from .toolbar import IToolbarModifier
from .utils import IDashboardUtils
from .utils import IUtils
from .views import ISecureLoginAllowedView
from .views import ITileView
from .widgets import IReCaptchaWidget



from castle.cms import ua
from castle.cms.interfaces import IDashboardUtils
from castle.cms.lockout import get_active_sessions
from castle.cms.lockout import SessionManager
from collective.elasticsearch.es import ElasticSearchCatalog
from elasticsearch import TransportError
from plone import api
from plone.app.blocks.interfaces import IBlocksTransformEnabled
from plone.protect.interfaces import IDisableCSRFProtection
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import IHideFromBreadcrumbs
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import implements
from zope.security import checkPermission

import socket


class Dashboard(BrowserView):

    def __call__(self):
        self.dashboard = self.get_dashboard()
        view = api.content.get_view(
            name='layout_view', context=self.dashboard, request=self.request)
        self.request.set('disable_border', 1)
        self.request.set('disable_plone.leftcolumn', 0)
        self.request.set('disable_plone.rightcolumn', 0)
        return view()

    def get_dashboard(self):
        write = False
        with api.env.adopt_roles(['Manager']):
            if 'dashboards' not in self.context.objectIds():
                write = True
                api.content.create(type='Folder', title='Dashboards', id='dashboards',
                                   container=self.context, exclude_from_nav=True)
            dashboards = self.context['dashboards']
            if not IHideFromBreadcrumbs.providedBy(dashboards):
                alsoProvides(dashboards, IHideFromBreadcrumbs)
            if api.content.get_state(obj=dashboards, default='Unknown') != 'published':
                write = True
                api.content.transition(obj=dashboards, transition='publish')

        member = api.user.get_current()
        user_id = member.getId()
        if user_id not in dashboards.objectIds():
            with api.env.adopt_roles(['Manager']):
                write = True
                # first make sure it is in allowed types...
                pts = api.portal.get_tool('portal_types')
                Folder = pts['Folder']
                if 'Dashboard' not in Folder.allowed_content_types:
                    allowed = list(Folder.allowed_content_types)
                    allowed.append('Dashboard')
                    Folder.allowed_content_types = tuple(allowed)

                aspect = ISelectableConstrainTypes(dashboards)
                if (aspect.getConstrainTypesMode() != 1 or
                        ['Dashboard'] != aspect.getImmediatelyAddableTypes()):
                    aspect.setConstrainTypesMode(1)
                    aspect.setImmediatelyAddableTypes(['Dashboard'])

                api.content.create(type='Dashboard', title='Dashboard', id=user_id,
                                   container=dashboards, exclude_from_nav=True)

        dashboard = dashboards[user_id]
        if dashboard.getOwner().getId() != user_id:
            with api.env.adopt_roles(['Manager']):
                write = True
                dashboard.changeOwnership(member.getUser(), recursive=False)
                dashboard.reindexObjectSecurity()
        if write:
            alsoProvides(self.request, IDisableCSRFProtection)

        return dashboard


class DashboardUtils(BrowserView):
    implements(IDashboardUtils)

    default_interests = (
        'Document',
        'News Item',
        'Link',
        'Folder',
        'UPLOAD',
        'MANAGE'
    )

    def __call__(self):
        alsoProvides(self.request, IBlocksTransformEnabled)

        self.site = api.portal.get()
        if (self.request.REQUEST_METHOD == 'POST' and
                self.request.form.get('removesession') == 'yes'):
            session_id = self.request.form.get('id')
            user = api.user.get_current()
            sm = SessionManager(api.portal.get(), self.request, user)
            sm.session_id = session_id
            sm.expire()
        self._user_cache = {}
        self.pas_member = getMultiAdapter(
            (self.site, self.request), name=u"pas_member")
        self.util = getMultiAdapter((self.site, self.request),
                                    name="castle-utils")
        self.sessions = self.get_open_sessions()

        self.has_add_permission = api.user.has_permission(
            'Add portal content', obj=self.site)

        return self

    def parse_ua(self, session):
        return '%s: %s' % ua.simple_detect(session['ua'])

    def get_open_sessions(self):
        keys_key = '%s-session-%s*' % (
            '-'.join(self.site.getPhysicalPath()[1:]),
            api.user.get_current().getId()
        )
        session_id = self.request.cookies.get('castle_session_id', None)
        sessions = []
        for session in get_active_sessions(keys_key):
            if session['id'] == session_id:
                continue
            sessions.append(session)
        return sessions

    def whois(self, ip):
        try:
            return socket.gethostbyaddr(ip)[0]
        except:
            pass

    def get_creator(self, item):
        if item.Creator in self._user_cache:
            return self._user_cache[item.Creator]
        info = self.pas_member.info(item.Creator)
        self._user_cache[item.Creator] = info
        return info

    def get_modifier(self, item):
        try:
            modifier = item.last_modified_by
            try:
                # to check if it is hashable and correct type
                foobar = modifier in self._user_cache  # noqa
            except:
                modifier = item.Creator
        except:
            modifier = item.Creator
        if modifier in self._user_cache:
            return self._user_cache[modifier]
        info = self.pas_member.info(modifier)
        self._user_cache[modifier] = info
        return info

    def _paging(self, query, name):
        try:
            start = int(self.request.form.get(name + '_start'))
        except:
            start = 0
        end = start + 30
        catalog = api.portal.get_tool('portal_catalog')
        # query['sort_limit'] = end
        results = catalog(**query)
        return {
            'start': start,
            'end': end,
            'results': results[start:end],
            'name': name,
            'total': len(results)
        }

    def get_recently_modified(self):
        query = dict(sort_on='modified', sort_order='reverse')
        return self._paging(query, 'modified')

    def get_recently_created(self):
        query = dict(sort_on='created', sort_order='reverse')
        return self._paging(query, 'created')

    def get_in_review(self):
        query = dict(sort_on='modified', review_state='pending', sort_order='reverse')
        return self._paging(query, 'review')

    def get_totals(self):
        query = {
            "size": 0,
            "aggregations": {
                "totals": {
                    "terms": {
                        "field": "portal_type"
                    }
                }
            }
        }
        portal_catalog = api.portal.get_tool('portal_catalog')
        try:
            es = ElasticSearchCatalog(portal_catalog)
            result = es.connection.search(
                index=es.index_name,
                doc_type=es.doc_type,
                body=query)
        except TransportError:
            return []

        return result['aggregations']['totals']['buckets']

    def _get_base_interest_query(self):
        return {
            "size": 0,
            "aggregations": {
                "totals": {
                    "terms": {
                        "field": "parent_folder"
                    }
                }
            }
        }

    def _make_query(self, query):
        portal_catalog = api.portal.get_tool('portal_catalog')
        try:
            es = ElasticSearchCatalog(portal_catalog)
            return es.connection.search(
                index=es.index_name,
                doc_type=es.doc_type,
                body=query)['aggregations']['totals']['buckets']
        except TransportError:
            return []

    def _get_creation_areas_of_interest(self, user_id):
        query = self._get_base_interest_query()
        query['aggregations']['totals']["aggregations"] = {
            "types": {
                "terms": {
                    "field": "portal_type"
                }
            }
        }
        query['query'] = {
            'filtered': {
                'filter': {
                    "and": [
                        {'term': {'Creator': user_id}}
                    ]
                },
                'query': {"match_all": {}}
            }
        }
        return self._make_query(query)

    def _get_contribution_areas_of_interest(self, user_id):
        query = self._get_base_interest_query()
        query['query'] = {
            'filtered': {
                'filter': {
                    "and": [
                        {'term': {'contributors': user_id}}
                    ]
                },
                'query': {"match_all": {}}
            }
        }
        return self._make_query(query)

    def is_root(self, obj):
        return IPloneSiteRoot.providedBy(obj)

    def get_path(self, obj):
        site_path = '/'.join(self.site.getPhysicalPath())
        obj_path = '/'.join(obj.getPhysicalPath())
        return obj_path[len(site_path):]

    def find_areas_of_interest(self):
        max_size = 6
        user_id = api.user.get_current().getId()
        site_path = '/'.join(self.site.getPhysicalPath())
        registry = getUtility(IRegistry)

        interests = []
        upload_found = False
        manage_site_found = False
        for result in self._get_creation_areas_of_interest(user_id):
            for tcount in result['types']['buckets'][:2]:  # max 2 per location
                pt = tcount['key']
                if pt in ('Image', 'File', 'Video', 'Audio'):
                    upload_found = True
                    continue
                if pt in ('Dashboard', 'Feed'):  # ignored types, we should check nice types
                    continue
                interests.append((tcount['key'], result['key'], result['doc_count']))
        if upload_found:
            interests.append(('UPLOAD', '', 1))

        for result in self._get_contribution_areas_of_interest(user_id):
            # since so many will have these areas, ignore
            if result['key'].split('/')[-1] in ('image-repository',
                                                'file-repository'):
                continue
            if result['key'] == site_path:
                manage_site_found = True
                # make it last
                interests.append(('MANAGE', result['key'], -1))
            else:
                interests.append(('MANAGE', result['key'], result['doc_count'] - 1))

        if len(interests) == 0:
            default_interests = list(self.default_interests[:])
            for idx, itype in enumerate(reversed(default_interests)):
                interests.append((itype, site_path, idx))
        elif len(interests) < 6:
            max_size = 6
            default_interests = list(self.default_interests[:])
            if upload_found:
                default_interests.remove('UPLOAD')
            if not manage_site_found:
                default_interests.remove('MANAGE')
            existing = dict([(i[0], i[1]) for i in interests])
            for itype in default_interests:
                if itype in existing and existing[itype] != site_path:
                    continue
                interests.append((itype, site_path, 0))

        interests = [i for i in reversed(sorted(interests, key=lambda x: x[2]))]

        # now to check access
        pts = api.portal.get_tool('portal_types')
        checked_interests = []
        for interest in interests:
            itype = interest[0]
            loc = interest[1]
            if loc != site_path:
                rel_path = loc[len(site_path):]
                folder = api.content.get(path=str(rel_path))
            else:
                folder = self.site

            if itype == 'MANAGE':
                if folder is None:
                    continue
                if checkPermission('cmf.ListFolderContents', folder):
                    checked_interests.append({
                        'type': itype,
                        'loc': folder
                    })
            elif itype == 'UPLOAD':
                file_location = registry.get('file_repo_location', '/file-repository')
                image_location = registry.get('image_repo_location', '/image-repository')
                video_location = registry.get('video_repo_location', '/video-repository')
                audio_location = registry.get('audio_repo_location', '/audio-repository')
                file_folder = api.content.get(path=str(file_location))
                image_folder = api.content.get(path=str(image_location))
                audio_folder = api.content.get(path=str(audio_location))
                video_folder = api.content.get(path=str(video_location))
                tocheck = (
                    ('File', file_folder),
                    ('Image', image_folder),
                    ('Audio', audio_folder),
                    ('Video', video_folder)
                )
                if file_folder is not None or image_folder is not None:
                    for pt_id, folder in tocheck:
                        if folder is None:
                            continue
                        pt = pts[pt_id]
                        if checkPermission(pt.add_permission, folder):
                            checked_interests.append({
                                'type': 'UPLOAD',
                                'loc': None
                            })
                            break
            else:
                if folder is None:
                    continue
                pt = pts[itype]
                if checkPermission(pt.add_permission, folder):
                    checked_interests.append({
                        'type': itype,
                        'loc': folder,
                        'pt': pt
                    })
        return checked_interests[:max_size]



import pycountry
from Acquisition import aq_parent
from castle.cms.fragments.interfaces import IFragmentsDirectory
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.component import getAllUtilitiesRegisteredFor
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.globalrequest import getRequest
from zope.interface import directlyProvides
from zope.interface import implementer
from zope.interface import implements
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
from plone.app.tiles.browser.edit import AcquirableDictionary
from plone.app.content.browser import vocabulary


# XXX needs updating in 5.1
try:
    vocabulary._permissions['plone.app.vocabularies.Groups'] = 'Modify portal content'
    vocabulary._permissions['castle.cms.vocabularies.EmailCategories'] = 'Modify portal content'
    vocabulary._permissions['plone.app.vocabularies.Keywords'] = 'View'
except:
    vocabulary.PERMISSIONS['plone.app.vocabularies.Groups'] = 'Modify portal content'
    vocabulary.PERMISSIONS['castle.cms.vocabularies.EmailCategories'] = 'Modify portal content'
    vocabulary.PERMISSIONS['plone.app.vocabularies.Keywords'] = 'View'
vocabulary._unsafe_metadata.append('last_modified_by')


_blacklist = (
    'editbar',
    'footer',
    'mainlinks',
    'sidenav',
    'sitetitle',
    'statusmessage',
    'toplinks',
    'topsharing',
    'mobilelinks',
    'mobilenav',
    'announcement',
    'search')

_dashboard_available = (
    'dashboard-statistics',
    'dashboard-welcome'
)


def AvailableFragments(context):
    # need to move import here since vocab module is used in interfaces
    from castle.cms.interfaces import IDashboard

    if isinstance(context, AcquirableDictionary):
        context = aq_parent(context)
    is_dash = IDashboard.providedBy(context)

    utils = getAllUtilitiesRegisteredFor(IFragmentsDirectory)
    all_frags = []
    request = getRequest()
    for util in utils:
        if util.layer is not None:
            if not util.layer.providedBy(request):
                continue
        all_frags.extend(util.list())

    terms = [SimpleVocabulary.createTerm('', '', 'Select fragment')]
    frags = []
    for frag in set(all_frags):
        if frag in _blacklist or (not is_dash and frag in _dashboard_available):
            continue
        if frag[0] == '_' or frag[-1] == '_':
            continue
        frags.append(frag)
    frags.sort()
    for frag in frags:
        terms.append(
            SimpleVocabulary.createTerm(frag, frag, frag.capitalize().replace('-', ' ')))
    return SimpleVocabulary(terms)
directlyProvides(AvailableFragments, IContextSourceBinder)


class RegistryValueSource(object):
    implements(IContextSourceBinder)

    def __init__(self, key_name, default=[]):
        self.key_name = key_name
        self.default = default

    def __call__(self, context):
        registry = getUtility(IRegistry)
        terms = []
        for value in registry.get(self.key_name, self.default):
            key = value
            if '|' in value:
                key, _, value = value.partition('|')
            terms.append(SimpleVocabulary.createTerm(key, key.encode('utf-8'), value))
        return SimpleVocabulary(terms)


@implementer(IVocabularyFactory)
class LocationsVocabularyFactory(object):

    def __call__(self, context):
        return RegistryValueSource('castle.allowed_locations')(context)


LocationsVocabulary = LocationsVocabularyFactory()


@implementer(IVocabularyFactory)
class MimeTypeVocabularyFactory(object):

    def __call__(self, context):
        catalog = api.portal.get_tool('portal_catalog')
        catalog.uniqueValuesFor('contentType')
        terms = []
        for value in catalog.uniqueValuesFor('contentType'):
            human = value
            if 'html' in value:
                human = 'HTML'
            elif value.split('/')[0] in ('audio', 'video', 'image'):
                human = value.split('/')[-1].upper()
            terms.append(SimpleVocabulary.createTerm(value, value.encode('utf-8'), human))
        return SimpleVocabulary(terms)

MimeTypeVocabulary = MimeTypeVocabularyFactory()


@implementer(IVocabularyFactory)
class EmailCategoryVocabularyFactory(object):

    def __call__(self, context):
        registry = getUtility(IRegistry)
        categories = registry.get('castle.subscriber_categories')
        terms = []

        for category in categories:
            terms.append(SimpleTerm(value=category, title=category))
        return SimpleVocabulary(terms)

EmailCategoryVocabulary = EmailCategoryVocabularyFactory()


BUSINES_TYPES = [
    'Restaurant',
    'AnimalShelter',
    'AutomotiveBusiness',
    'ChildCare',
    'DryCleaningOrLaundry',
    'EmergencyService',
    'EmploymentAgency',
    'EntertainmentBusiness',
    'FinancialService',
    'FoodEstablishment',
    'GovernmentOffice',
    'HealthAndBeautyBusiness',
    'HomeAndConstructionBusiness',
    'InternetCafe',
    'LegalService',
    'Library',
    'LodgingBusiness',
    'MedicalOrganization',
    'Organization',
    'ProfessionalService',
    'RadioStation',
    'RealEstateAgent',
    'RecyclingCenter',
    'SelfStorage',
    'ShoppingCenter',
    'SportsActivityLocation',
    'Store',
    'TelevisionStation',
    'TouristInformationCenter',
    'TravelAgency',
    'Airline',
    'Corporation',
    'EducationalOrganization',
    'GovernmentOrganization',
    'LocalBusiness',
    'NGO',
    'PerformingGroup',
    'SportsOrganization',
    'AutoPartsStore',
    'BikeStore',
    'BookStore',
    'ClothingStore',
    'ComputerStore',
    'ConvenienceStore',
    'DepartmentStore',
    'ElectronicsStore',
    'Florist',
    'FurnitureStore',
    'GardenStore',
    'GroceryStore',
    'HardwareStore',
    'HobbyShop',
    'HomeGoodsStore',
    'JewelryStore',
    'LiquorStore',
    'MensClothingStore',
    'MobilePhoneStore',
    'MovieRentalStore',
    'MusicStore',
    'OfficeEquipmentStore',
    'OutletStore',
    'PawnShop',
    'PetStore',
    'ShoeStore',
    'SportingGoodsStore',
    'TireShop',
    'ToyStore',
    'WholesaleStore'
]
BUSINES_TYPES.sort()
BusinessTypesVocabulary = SimpleVocabulary([
    SimpleVocabulary.createTerm('', '', 'n/a'),
] + [
    SimpleVocabulary.createTerm(name, name, name)
    for name in BUSINES_TYPES
])
del BUSINES_TYPES


BAD_TYPES = ("ATBooleanCriterion", "ATDateCriteria", "ATDateRangeCriterion",
             "ATListCriterion", "ATPortalTypeCriterion", "ATReferenceCriterion",
             "ATSelectionCriterion", "ATSimpleIntCriterion", "Plone Site",
             "ATSimpleStringCriterion", "ATSortCriterion", "TempFolder",
             "ATCurrentAuthorCriterion", "ATPathCriterion",
             "ATRelativePathCriterion", "Pad", 'Comment', 'Link')


class ReallyUserFriendlyTypesVocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        site = getSite()
        ttool = getToolByName(site, 'portal_types', None)
        if ttool is None:
            return SimpleVocabulary([])

        items = [(ttool[t].Title(), t)
                 for t in ttool.listContentTypes()
                 if t not in BAD_TYPES]
        items.sort()
        items = [SimpleTerm(i[1], i[1], i[0]) for i in items]
        return SimpleVocabulary(items)

ReallyUserFriendlyTypesVocabularyFactory = ReallyUserFriendlyTypesVocabulary()


class CountriesVocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        items = []
        for country in pycountry.countries:
            items.append(SimpleTerm(country.alpha2, country.alpha2, country.name))
        return SimpleVocabulary(items)


CountriesVocabularyFactory = CountriesVocabulary()



from collective.elasticsearch.interfaces import IElasticSearchLayer
from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from plone.app.mosaic.interfaces import IMosaicLayer
from plone.app.tiles.interfaces import ITilesFormLayer
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class ICastleLayer(IDefaultBrowserLayer, IMosaicLayer, ITilesFormLayer,
                   IPloneAppContenttypesLayer, IElasticSearchLayer):
    pass


class IVersionViewLayer(ICastleLayer):
    """
    when viewing a version of an object
    """



# -*- coding: utf-8 -*-
from zope.component import getAdapters
from plone.folder.interfaces import IOrdering
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary
from zope.interface import implements
from plone.app.contenttypes import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from plone.dexterity.interfaces import IDexterityContainer
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider
from zope import schema


class AvailableOrderSource(object):
    implements(IContextSourceBinder)

    ignored = (
        'partial',
    )

    labels = {
        '': 'New items added to bottom',
        'reversed': 'New items add to top'
    }

    def __init__(self):
        pass

    def get_label(self, name):
        if name in self.labels:
            return self.labels[name]
        return name.capitalize()

    def __call__(self, context):
        terms = []
        for name in sorted([name for name, _ in getAdapters((context,), provided=IOrdering)]):
            if name in self.ignored:
                continue
            terms.append(SimpleVocabulary.createTerm(name, name, self.get_label(name)))
        return SimpleVocabulary(terms)


@provider(IFormFieldProvider)
class IFolderOrder(model.Schema):

    model.fieldset('settings', label=_(u"Settings"),
                   fields=['folder_order'])

    folder_order = schema.Choice(
        title=_(u'Folder order'),
        description=u"Order in which new content is added to container",
        required=True,
        default=u'',
        source=AvailableOrderSource()
    )


@implementer(IFolderOrder)
@adapter(IDexterityContainer)
class FolderOrder(object):

    def __init__(self, context):
        self.context = context

    def _set_folder_order(self, value):
        if isinstance(value, unicode):
            value = value.encode('utf8')
        self.context._ordering = value

    def _get_folder_order(self):
        if hasattr(self.context, '_ordering'):
            return self.context._ordering.decode('utf8')

    folder_order = property(_get_folder_order, _set_folder_order)



from castle.cms.interfaces import IReferenceNamedImage
from castle.cms.interfaces import IVersionViewLayer
from DateTime import DateTime
from lxml.html import fromstring
from lxml.html import tostring
from plone import api
from plone.app.layout.viewlets.content import ContentHistoryView as BaseContentHistoryView
from plone.app.layout.viewlets.content import HistoryByLineView
from plone.memoize.view import memoize
from plone.protect.interfaces import IDisableCSRFProtection
from Products.CMFEditions.browser.diff import DiffView
from Products.CMFPlone.browser.syndication.adapters import SearchFeed
from Products.CMFPlone.interfaces.syndication import IFeedItem
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import noLongerProvides


class HistoryView(HistoryByLineView):
    index = ViewPageTemplateFile('templates/history_view.pt')

    def show(self):
        """
        just use permission checking
        """
        return True

    def show_history(self):
        """
        see comment for pervious method
        """
        return True


class ContentHistoryView(BaseContentHistoryView):
    index = ViewPageTemplateFile("templates/content_history.pt")


class HistoryVersionView(DiffView):
    template = ViewPageTemplateFile("templates/history_version.pt")

    def getContent(self, version):
        alsoProvides(self.request, IVersionViewLayer)
        feed = SearchFeed(api.portal.get())
        adapter = queryMultiAdapter((version, feed), IFeedItem)
        content = adapter.render_content_core().strip()
        noLongerProvides(self.request, IVersionViewLayer)

        if not content:
            # try old fashioned way... bah!
            content = version.restrictedTraverse(version.defaultView())()
            dom = fromstring(content)
            return ''.join([tostring(el) for el in dom.cssselect('#content-core > *')])
        else:
            dom = fromstring(content)
            return ''.join([tostring(el) for el in dom.cssselect('[data-panel] > *')])

    @property
    @memoize
    def versions(self):
        version_history = []
        rt = api.portal.get_tool("portal_repository")
        history = rt.getHistoryMetadata(self.context)
        retrieve = history.retrieve
        getId = history.getVersionId

        # Count backwards from most recent to least recent
        for i in xrange(history.getLength(countPurged=False) - 1, -1, -1):
            version_id = getId(i, countPurged=False)
            data = retrieve(i, countPurged=False)
            meta = data["metadata"]["sys_metadata"]
            version_history.append({
                'dt': DateTime(meta['timestamp']),
                'comments': meta['comment'],
                'version_id': version_id
            })

        return version_history

    def __call__(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        self.version_info = None
        for version in self.versions:
            if str(version['version_id']) == self.request.form.get('version'):
                self.version_info = version
        return self.template()

    def get_referenced_image(self, obj):
        if IReferenceNamedImage.providedBy(obj.image):
            catalog = api.portal.get_tool('portal_catalog')
            brains = catalog.unrestrictedSearchResults(UID=self.context.image.reference)
            if len(brains) > 0:
                return brains[0].getObject()



from AccessControl import Unauthorized
from boto.s3.prefix import Prefix
from castle.cms import archival
from castle.cms.files import aws
from DateTime import DateTime
from plone import api
from plone.app.uuid.utils import uuidToObject
from Products.Five import BrowserView
from zope.component import getMultiAdapter

import json


class BaseView(BrowserView):

    @property
    def enabled(self):
        return (api.portal.get_registry_record('castle.archival_enabled') and
                api.portal.get_registry_record('castle.aws_s3_bucket_name') and
                api.portal.get_registry_record('castle.aws_s3_key') and
                api.portal.get_registry_record('castle.aws_s3_secret') and
                api.portal.get_registry_record('plone.public_url'))


class Review(BaseView):

    def dump(self, item):
        return {
            'title': item.Title,
            'url': item.getURL(),
            'uid': item.UID,
            'modified': item.modified.pCommonZ()
        }

    def items(self):
        return [self.dump(i) for i in archival.getContentToArchive(30)]

    def json_dump(self):
        return json.dumps(self.items())

    def __call__(self):
        extend = self.request.form.get('extend')
        if extend:
            obj = uuidToObject(extend)
            obj.setModificationDate(DateTime())
            obj.reindexObject(idxs=['modified'])
            return self.json_dump()
        return self.index()


class AWSApi(object):
    def __init__(self, site, request):
        self.site = site
        self.request = request
        bucket_name = api.portal.get_registry_record('castle.aws_s3_bucket_name')
        self.s3_conn, self.bucket = aws.get_bucket(bucket_name)
        self.archive_storage = archival.Storage(site)

    def __call__(self):
        method = self.request.form.get('method')
        if method == 'list':
            return self.list()
        elif method == 'get':
            return self.get()
        elif method == 'save':
            return self.save()
        elif method == 'delete':
            return self.delete()
        elif method == 'deletegroup':
            return self.deletegroup()

    def get(self):
        key_name = archival.CONTENT_KEY_PREFIX + self.request.form.get('path')
        key = self.bucket.get_key(key_name)
        if 'html' not in key.content_type:
            raise Exception('Must be html...')
        return {
            'data': key.read()
        }

    def deletegroup(self):
        authenticator = getMultiAdapter((self.site, self.request), name=u"authenticator")
        # manually provide csrf here...
        if not authenticator.verify():
            raise Unauthorized

        paths = json.loads(self.request.form.get('paths'))

        # path can be folder or item... get all items and then do multi delete from aws api
        todelete = []
        for path in paths:
            if path.endswith('/'):
                # this is a folder, delete everything in it
                base_path = archival.CONTENT_KEY_PREFIX + path
                base_path = base_path.replace('//', '/').rstrip('/') + '/'
                for key in self.bucket.list(base_path):
                    if isinstance(key, Prefix):
                        continue
                    todelete.append(key.name)
                    self.delete_archive(key.name[len(archival.CONTENT_KEY_PREFIX):])
            else:
                self.delete_archive(path)
                todelete.append(archival.CONTENT_KEY_PREFIX + path)

        self.bucket.delete_keys(todelete)

    def delete(self):
        path = self.request.form.get('path')
        key_name = archival.CONTENT_KEY_PREFIX + path
        authenticator = getMultiAdapter((self.site, self.request), name=u"authenticator")
        # manually provide csrf here...
        if not authenticator.verify():
            raise Unauthorized

        key = self.bucket.get_key(key_name)
        if key:
            self.bucket.delete_key(key_name)
        self.delete_archive(path)

    def delete_archive(self, path):
        path = '/' + path.strip('/')
        try:
            uid = self.archive_storage.path_to_uid[path]
        except KeyError:
            return
        try:
            del self.archive_storage.archives[uid]
        except:
            pass

    def save(self):
        key_name = archival.CONTENT_KEY_PREFIX + self.request.form.get('path')
        authenticator = getMultiAdapter((self.site, self.request), name=u"authenticator")
        # manually provide csrf here...
        if not authenticator.verify():
            raise Unauthorized

        content = self.request.form.get('value')
        key = self.bucket.get_key(key_name)
        key.set_contents_from_string(content, headers={
            'Content-Type': 'text/html; charset=utf-8'
        }, replace=True)
        key.make_public()

    def list(self):
        result = []
        base_path = archival.CONTENT_KEY_PREFIX + self.request.form.get('path', '')
        base_path = base_path.replace('//', '/').rstrip('/') + '/'
        for key in self.bucket.list(base_path, '/'):
            path = key.name[len(archival.CONTENT_KEY_PREFIX):]
            result.append({
                'path': path,
                'id': path.rstrip('/').split('/')[-1],
                'is_folder': isinstance(key, Prefix),
                'url': 'https://{host}/{bucket}/{key}'.format(
                    host=self.s3_conn.server_name(),
                    bucket=self.bucket.name,
                    key=archival.CONTENT_KEY_PREFIX + path)
            })
        return result


class Manage(BaseView):

    def __call__(self):
        if self.request.form.get('api'):
            self.request.response.setHeader('Content-type', 'application/json')
            return json.dumps(AWSApi(self.context, self.request)())
        return self.index()



from plone.namedfile.interfaces import INamedImage
from zope.interface import Attribute
from OFS.interfaces import IApplication
from plone.app.contenttypes.interfaces import IFile
from plone.supermodel import model
from Products.CMFPlone.interfaces import IHideFromBreadcrumbs
from zope.interface import Interface


class ICastleApplication(IApplication):
    pass


class IDashboard(IHideFromBreadcrumbs):
    pass


class IMedia(model.Schema, IFile):
    pass


class IVideo(IMedia):
    pass


class IAudio(IMedia):
    pass


class ITrashed(Interface):
    """
    marker for object that is in the trash
    """


class IHasDefaultImage(Interface):
    """
    marker interface for content to specify that it has default image
    when a manual image is not specified
    """

class IReferenceNamedImage(INamedImage):
    reference = Attribute('')



from castle.cms import subscribe
from castle.cms import utils
from castle.cms.browser.utils import Utils
from collective.celery import task
from plone import api
from urllib import urlencode
from zope.globalrequest import getRequest


@task.as_admin()
def send_email(*args, **kwargs):
    utils.send_email(*args, **kwargs)


@task.as_admin()
def send_email_to_subscribers(subject, html, categories=None):
    _utils = Utils(api.portal.get(), getRequest())
    public_url = _utils.get_public_url()

    check_categories = (categories is not None and len(categories) != 0)

    for subscriber in subscribe.all():
        if check_categories:
            # If there's no chosen categories, they recieve everything
            if 'categories' in subscriber and len(subscriber['categories']) > 0:

                # make sure that this message falls under one of their categories
                if len(categories.intersection(subscriber['categories'])) == 0:
                    return

        query = urlencode({
            'email': subscriber.get('email'),
            'code': subscriber.get('code')
            })

        unsubscribe_url = '%s/@@unsubscribe?%s' % (
            public_url.rstrip('/'),
            query)

        change_url = '%s/@@changesubscription?%s' % (
            public_url.rstrip('/'),
            query)

        unsubscribe_url = unsubscribe_url.encode('utf8')
        change_url = change_url.encode('utf8')

        html = html.replace('{{unsubscribe_url}}', unsubscribe_url)
        html = html.replace('{{change_url}}', change_url)

        utils.send_email([subscriber.get('email')], subject, html)



# implements site subscription technology
from persistent.mapping import PersistentMapping
from time import time

from BTrees.OOBTree import OOBTree
from castle.cms.utils import make_random_key
from plone import api


class InvalidEmailException(KeyError):
    pass


class InvalidCodeException(Exception):
    pass


class SubscriptionStorage(object):

    def __init__(self):
        self.site = api.portal.get()
        try:
            self._data = self.site._subscribers
        except AttributeError:
            self._data = OOBTree()
            self.site._subscribers = self._data

    def add(self, email, data=None):
        email = email.lower()
        if data is None:
            data = {}
        data.update({
            'created': time(),
            'code': make_random_key(100),
            'confirmed': False,
            'phone_number_confirmed': False,
            'email': email
        })
        self._data[email] = PersistentMapping(data)
        return data

    def remove(self, email):
        if email.lower() in self._data:
            del self._data[email.lower()]

    def get(self, email):
        return self._data.get(email.lower())


def get(email):
    storage = SubscriptionStorage()
    return storage.get(email)


def remove(email):
    storage = SubscriptionStorage()
    return storage.remove(email)


def register(email, data):
    storage = SubscriptionStorage()
    return storage.add(email, data)


def confirm(email, code):
    storage = SubscriptionStorage()
    subscriber = storage.get(email)
    if not subscriber:
        raise InvalidEmailException(email)
    if subscriber['code'] != code:
        raise InvalidCodeException()
    subscriber['confirmed'] = True


def confirm_phone_number(email, code):
    storage = SubscriptionStorage()
    subscriber = storage.get(email)
    if not subscriber:
        raise InvalidEmailException(email)
    if len(code) < 6:
        raise InvalidCodeException()
    if not subscriber['code'].endswith(code):
        raise InvalidCodeException()
    subscriber['phone_number_confirmed'] = True


def get_subscriber(email):
    storage = SubscriptionStorage()
    return storage.get(email)


def all():
    storage = SubscriptionStorage()
    for subscriber in storage._data.values():
        if subscriber.get('confirmed'):
            yield subscriber


def get_phone_numbers():
    numbers = []
    storage = SubscriptionStorage()
    for subscriber in storage._data.values():
        number = subscriber.get('phone_number')
        if (number and subscriber.get('phone_number_confirmed', False)
                and subscriber.get('confirmed')):
            numbers.append(number)
    return numbers


def get_email_addresses():
    storage = SubscriptionStorage()
    result = []
    for email, subscriber in storage._data.items():
        if subscriber.get('confirmed'):
            result.append(email)

    return result



from castle.cms.widgets import MapPointsFieldWidget
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import validate_json
from zope import schema
from zope.interface import alsoProvides


class ILocation(model.Schema):

    model.fieldset(
        'categorization',
        fields=['locations', 'coordinates'],
    )

    form.widget('locations', AjaxSelectFieldWidget,
                vocabulary='castle.cms.vocabularies.Locations')
    locations = schema.List(
        title=u'Locations',
        description=u'Locations to be referred to with this content',
        required=False,
        value_type=schema.TextLine()
    )

    form.widget(coordinates=MapPointsFieldWidget)
    coordinates = schema.Text(
        title=u'Coordinates',
        description=u'Providing coordinates to be able to map this content. '
                    u'You can select more than one set of coordinates',
        default=u'[]',
        constraint=validate_json,
        required=False
    )


alsoProvides(ILocation, IFormFieldProvider)



from AccessControl import getSecurityManager
from Acquisition import aq_parent
from castle.cms import authentication
from castle.cms import cache
from castle.cms.interfaces import ICastleApplication
from collective.elasticsearch.es import ElasticSearchCatalog  # noqa
from OFS.CopySupport import _cb_decode
from OFS.CopySupport import _cb_encode
from OFS.CopySupport import CopyError
from OFS.CopySupport import eInvalid
from OFS.CopySupport import eNoData
from plone import api
from plone.keyring.interfaces import IKeyManager
from plone.namedfile.interfaces import INamedField
from plone.rfc822.interfaces import IPrimaryFieldInfo
from plone.session import tktauth
from plone.transformchain.interfaces import ITransform
from time import time
from ZODB.POSException import POSKeyError
from zope.component import getGlobalSiteManager
from zope.component import queryUtility
from zope.interface import implementer


def HideSiteLayoutFields_update(self):
    """
    we don't want to hide these fields
    """
    return


def MosaicRegistry_parseRegistry(self):
    cache_key = '%s-mosaic-registry' % '/'.join(api.portal.get().getPhysicalPath()[1:])
    try:
        return cache.get(cache_key)
    except KeyError:
        result = self._old_parseRegistry()
        cache.set(cache_key, result, 60 * 2)  # cache for 2 minutes
        return result


if not hasattr(ElasticSearchCatalog, 'original_searchResults'):
    ElasticSearchCatalog.original_searchResults = ElasticSearchCatalog.searchResults
    def searchResultsTrashed(self, REQUEST=None, check_perms=False, **kw):
        if 'trashed' not in kw:
            kw['trashed'] = False
        return self.original_searchResults(REQUEST, check_perms, **kw)
    ElasticSearchCatalog.searchResults = searchResultsTrashed


def Content_addCreator(self, creator=None):
    """ Add creator to Dublin Core creators.
    """
    if len(self.creators) > 0:
        # do not add creator if one is already set
        return

    if creator is None:
        user = getSecurityManager().getUser()
        creator = user and user.getId()

    # call self.listCreators() to make sure self.creators exists
    if creator and creator not in self.listCreators():
        self.creators = self.creators + (creator, )


def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):
    """
    override normal paste action to see if we're looking at the paste
    data being stored in cache
    """
    if cb_copy_data is not None:
        cp = cb_copy_data
    elif REQUEST is not None and '__cp' in REQUEST:
        cp = REQUEST['__cp']
    else:
        cp = None
    if cp is None:
        raise CopyError(eNoData)

    try:
        op, mdatas = _cb_decode(cp)
    except:
        raise CopyError(eInvalid)

    try:
        if mdatas[0][0].startswith('cache:'):
            cache_key = mdatas[0][0].replace('cache:', '')
            new_mdatas = cache.get(cache_key)
            cdata = _cb_encode((op, new_mdatas))
            return self._old_manage_pasteObjects(cdata)
    except IndexError:
        pass
    return self._old_manage_pasteObjects(cb_copy_data, REQUEST)


@implementer(ITransform)
class NoopTransform(object):

    order = 8800

    def __init__(self, published, request):
        self.published = published
        self.request = request

    def transformString(self, result, encoding):
        return None
    transformUnicode = transformIterable = transformString


def AppInitializer_initialize(self):
    self._old_initialize()
    app = self.app[0]
    authentication.install_acl_users(app, self.commit)


def SessionPlugin_validateTicket(self, ticket, now=None):
    if now is None:
        now = time()
    if self._shared_secret is not None:
        ticket_data = tktauth.validateTicket(
            self._shared_secret,
            ticket,
            timeout=self.timeout,
            now=now,
            mod_auth_tkt=self.mod_auth_tkt
        )
    else:
        ticket_data = None
        parent = aq_parent(aq_parent(self))

        is_root = ICastleApplication.providedBy(parent)
        if is_root:
            manager = getGlobalSiteManager().queryUtility(IKeyManager)
        else:
            manager = queryUtility(IKeyManager)

        if manager is None:
            return None

        for secret in manager[u"_system"]:
            if secret is None:
                continue
            ticket_data = tktauth.validateTicket(
                secret,
                ticket,
                timeout=self.timeout,
                now=now,
                mod_auth_tkt=self.mod_auth_tkt
            )
            if ticket_data is not None:
                break
    return ticket_data



from castle.cms.tiles.base import BaseTile
from castle.cms.widgets import FocalPointSelectFieldWidget
from castle.cms.widgets import ImageRelatedItemFieldWidget
from castle.cms.widgets import RelatedItemFieldWidget
from plone.autoform import directives as form
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from plone.tiles.interfaces import IPersistentTile
from Products.CMFPlone.interfaces.controlpanel import IImagingSchema
from zope import schema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.globalrequest import getRequest
from zope.interface import implements
from zope.interface import Invalid
from zope.interface import invariant
from zope.interface import provider
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


@provider(IContextSourceBinder)
def image_scales(context):
    values = []
    registry = getUtility(IRegistry)
    settings = registry.forInterface(IImagingSchema,
                                     prefix="plone",
                                     check=False)
    values.append(SimpleTerm('', '', 'Original'))
    for allowed_size in settings.allowed_sizes:
        name = allowed_size.split()[0]
        values.append(SimpleTerm(name, name, allowed_size))
    return SimpleVocabulary(values)


class ImageTile(BaseTile):
    implements(IPersistentTile)

    def render(self):
        return self.index()

    def get_image(self):
        image = self.data.get('image')
        if not image:
            return
        return self.utils.get_object(self.data['image'][0])

    def get_link(self):
        link = self.data.get('link')
        if not link:
            return
        try:
            return self.utils.get_object(link[0])
        except:
            pass


class IImageTileSchema(model.Schema):

    form.widget(image=ImageRelatedItemFieldWidget)
    image = schema.List(
        title=u"Image",
        description=u"Reference image on the site.",
        required=True,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    @invariant
    def validate_image(data):
        if data.image and len(data.image) != 1:
            raise Invalid("Must select 1 image")
        if data.image:
            utils = getMultiAdapter((getSite(), getRequest()),
                                    name="castle-utils")
            obj = utils.get_object(data.image[0])
            if obj.portal_type != 'Image':
                raise Invalid('Must provide image file')

    scale = schema.Choice(
        title=u'Scale',
        required=True,
        source=image_scales,
        default=u'large'
    )

    display_type = schema.Choice(
        title=u'Display type',
        required=True,
        default=u'natural',
        vocabulary=SimpleVocabulary([
            SimpleTerm('natural', 'natural', u'Natural'),
            SimpleTerm('fullwidth', 'fullwidth', u'Natural(Full width)'),
            SimpleTerm('portrait', 'portrait', u'Portrait'),
            SimpleTerm('landscape', 'landscape', u'Landscape'),
            SimpleTerm('square', 'square', u'Square'),
            SimpleTerm('short', 'short', u'Short'),
        ])
    )

    caption = schema.TextLine(
        title=u'Caption',
        description=u'The caption shows under the image. This is different than the summary'
                    u' field on the image',
        required=False
    )

    form.widget(override_focal_point=FocalPointSelectFieldWidget)
    override_focal_point = schema.Text(
        title=u'Override Focal point',
        default=u'',
        required=False
    )

    form.widget(link=RelatedItemFieldWidget)
    link = schema.List(
        title=u"Link",
        description=u"Content to link this image to.",
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    @invariant
    def validate_link(data):
        if data.link and len(data.link) != 1:
            raise Invalid("Must select 1 link only")



import json

from castle.cms.tiles.base import BaseImagesTile
from castle.cms.widgets import ImageRelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary


class SliderTile(BaseImagesTile):

    @property
    def pattern_options(self):
        opts = {
            'animation': self.data.get('animation', 'fade'),
            'controlNav': self.data.get('controlNav', True),
            'directionNav': self.data.get('directionNav', True),
            'slideshowSpeed': self.data.get('slideshowSpeed', 7000),
            'animationSpeed': self.data.get('animationSpeed', 600)
        }
        return json.dumps(opts)

    def render(self):
        return self.index()

    @property
    def height(self):
        return self.data.get('height', 250)


class ISliderTileSchema(model.Schema):

    form.widget(images=ImageRelatedItemsFieldWidget)
    images = schema.List(
        title=u"Images",
        description=u"Select images or folders of images to display in slider."
                    u" If the image has a related item selected, that "
                    u"related item will be used for the link and description "
                    u"for the slide display",
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    animation = schema.Choice(
        title=u'Animation',
        vocabulary=SimpleVocabulary([
            SimpleVocabulary.createTerm('fade', 'fade', 'Fade'),
            SimpleVocabulary.createTerm('slide', 'slide', 'Slide'),
        ]),
        default='fade'
    )

    controlNav = schema.Bool(
        title=u'Control nav',
        default=True)

    directionNav = schema.Bool(
        title=u'Direction nav',
        default=True)

    slideshowSpeed = schema.Int(
        title=u'Cycle speed',
        description=u'In milliseconds',
        default=7000)

    animationSpeed = schema.Int(
        title=u'Animation speed',
        description=u'In milliseconds',
        default=600)



from AccessControl import getSecurityManager
from AccessControl import Unauthorized
from Acquisition import aq_inner
from Acquisition import aq_parent
from castle.cms import cache
from castle.cms import tasks
from castle.cms import trash
from castle.cms.utils import get_paste_data
from castle.cms.utils import is_max_paste_items
from OFS.CopySupport import _cb_encode
from OFS.CopySupport import cookie_path
from OFS.CopySupport import CopyError
from OFS.Moniker import Moniker
from plone import api
from plone.app.content.browser.contents import FolderContentsView as BaseFolderContentsView
from plone.app.content.browser.contents import copy
from plone.app.content.browser.contents import cut
from plone.app.content.browser.contents import delete
from plone.app.content.browser.contents import paste
from plone.app.content.browser.contents import rename
from plone.app.content.interfaces import IStructureAction
from plone.locking.interfaces import ILockable
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.log import logger
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from uuid import uuid4
from ZODB.POSException import ConflictError
from zope.component import getMultiAdapter
from zope.container.interfaces import INameChooser
from zope.event import notify
from zope.interface import implementer
from zope.lifecycleevent import ObjectModifiedEvent

import json
import transaction


try:
    from plone.app.content.browser.vocabulary import DEFAULT_PERMISSION_SECURE
except ImportError:
    DEFAULT_PERMISSION_SECURE = 'Modify portal content'


class CutActionView(cut.CutActionView):
    def finish(self):
        oblist = []
        for ob in self.oblist:
            if ob.wl_isLocked():
                self.errors.append(_(u'${title} is being edited and cannot be cut.',
                                     mapping={u'title': self.objectTitle(ob)}))
                continue
            if not ob.cb_isMoveable():
                self.errors.append(_(u'${title} is being edited and can not be cut.',
                                     mapping={u'title': self.objectTitle(ob)}))
                continue
            m = Moniker(ob)
            oblist.append(m.dump())
        if len(oblist) > 20 and cache.redis_installed():
            cache_key = str(uuid4())
            cache.set(cache_key, oblist, expire=60 * 60 * 24)
            cp = (1, [['cache:' + cache_key]])
        else:
            cp = (1, oblist)
        cp = _cb_encode(cp)
        resp = self.request.response
        resp.setCookie('__cp', cp, path='%s' % cookie_path(self.request))
        self.request['__cp'] = cp


class CopyActionView(copy.CopyActionView):
    def finish(self):
        oblist = []
        for ob in self.oblist:
            if not ob.cb_isCopyable():
                self.errors.append(_(u'${title} cannot be copied.',
                                     mapping={u'title': self.objectTitle(ob)}))
                continue
            m = Moniker(ob)
            oblist.append(m.dump())
        if len(oblist) > 20 and cache.redis_installed():
            cache_key = str(uuid4())
            cache.set(cache_key, oblist, expire=60 * 60 * 24)
            cp = (0, [['cache:' + cache_key]])
        else:
            cp = (0, oblist)
        cp = _cb_encode(cp)
        resp = self.request.response
        resp.setCookie('__cp', cp, path='%s' % cookie_path(self.request))
        self.request['__cp'] = cp


class PasteActionView(paste.PasteActionView):

    def paste_async(self, paste_data):
        self.errors = ['']
        self.failure_msg = """You have selected to paste %i items which is a large amount.
This action can take a long time to accomplish. We will email you
when the content is done being moved.""" % paste_data['count']
        tasks.paste_items.delay(
            self.request.form['folder'], paste_data['op'],
            paste_data['mdatas'])
        return self.json({
            'status': 'success',
            'msg': {
                'label': 'INFO',
                'text': self.failure_msg
            }
        })

    def copy_error(self):
        self.errors = ['']
        self.failure_msg = """There was an error pasting the data. It's possible
the number of items you have selected is larger than what the browser allows.
Try pasting a smaller number of items."""
        return self.json({
            'status': 'failure',
            'msg': {
                'label': 'WARNING',
                'type': 'warning',
                'text': self.failure_msg
            }
        })

    def __call__(self):
        try:
            paste_data = get_paste_data(self.request)
        except CopyError:
            return self.copy_error()
        if is_max_paste_items(paste_data):
            return self.paste_async(paste_data)

        return super(PasteActionView, self).__call__()


class DeleteActionView(delete.DeleteActionView):

    delete_warning = """
<div class="portalMessage error">
<b>WARNING</b>:
<p>You have selected to delete %i items which is a large amount.
This action can take a long time to accomplish. We will email you
when the content is done being deleted.</p>
<p>
<b>Link integrity checks can not be done on delete this large of a batch.</b>
</p>
</div>"""

    failure_msg_template = """
You have selected to delete %i items which is a large amount.
This action can take a long time to accomplish. We will email you
when the content is done being deleted."""

    def delete_async(self, uids, count):
        self.errors = ['']
        self.failure_msg = self.failure_msg_template % count
        tasks.delete_items.delay(uids)
        return self.json({
            'status': 'success',
            'msg': {
                'label': 'WARNING',
                'type': 'warning',
                'text': self.failure_msg
            }
        })

    def action(self, obj):
        parent = obj.aq_inner.aq_parent

        lockable = ILockable(obj, None)
        if lockable and lockable.locked():
            lockable.clear_locks()

        try:
            parent.manage_delObjects(obj.getId(), self.request)
        except Unauthorized:
            self.errors.append(
                _(u'You are not authorized to delete ${title}.',
                    mapping={u'title': self.objectTitle(self.dest)}))

    def __call__(self):
        selection = self.get_selection()
        over = False
        count = len(selection)
        if count < 100:
            catalog = api.portal.get_tool('portal_catalog')
            paths = []
            for brain in catalog(UID=selection):
                paths.append(brain.getPath())
            count = len(catalog(path={'query': paths, 'depth': -1}))
            if count > 100:
                over = True
        else:
            over = True

        if not over:
            return super(DeleteActionView, self).__call__()

        # special handling from here on so we can do async handling
        if self.request.form.get('render') == 'yes':
            self.request.response.setHeader('Content-Type', 'application/json')
            return json.dumps({
                'html': self.delete_warning % count
            })
        else:
            return self.delete_async(selection, count)


class TrashActionView(delete.DeleteActionView):
    success_msg = 'Successfully moved items to recycle bin'
    failure_msg = 'Failed to move items to recycle bin'

    def action(self, obj):
        trash.object(obj)


@implementer(IStructureAction)
class TrashAction(delete.DeleteAction):

    template = ViewPageTemplateFile('templates/fc-trash.pt')
    order = 4

    def get_options(self):
        return {
            'title': 'Recycle bin',
            'id': 'recycle',
            'icon': 'trash',
            'context': 'default',
            'url': self.context.absolute_url() + '/@@fc-trash',
            'form': {
                'title': 'Move selected items to recycle bin',
                'submitText': 'Yes',
                'submitContext': 'danger',
                'template': self.template(),
                'closeText': 'No',
                'dataUrl': self.context.absolute_url() + '/@@fc-trash'
            }
        }


class RenameActionView(rename.RenameActionView):
    success_msg = _('Items renamed')
    failure_msg = _('Failed to rename all items')

    def __call__(self):
        self.errors = []
        self.protect()
        context = aq_inner(self.context)

        catalog = getToolByName(context, 'portal_catalog')
        mtool = getToolByName(context, 'portal_membership')

        missing = []
        for key in self.request.form.keys():
            if not key.startswith('UID_'):
                continue
            index = key.split('_')[-1]
            uid = self.request.form[key]
            brains = catalog(UID=uid)
            if len(brains) == 0:
                missing.append(uid)
                continue
            obj = brains[0].getObject()
            title = self.objectTitle(obj)
            if not mtool.checkPermission('Copy or Move', obj):
                self.errors(_(u'Permission denied to rename ${title}.',
                              mapping={u'title': title}))
                continue

            sp = transaction.savepoint(optimistic=True)

            newid = self.request.form['newid_' + index].encode('utf8')
            newtitle = self.request.form['newtitle_' + index]

            lockable = ILockable(obj, None)
            if lockable:
                lockable.clear_locks()

            try:
                obid = obj.getId()
                title = obj.Title()
                change_title = newtitle and title != newtitle
                if change_title:
                    getSecurityManager().validate(obj, obj, 'setTitle',
                                                  obj.setTitle)
                    obj.setTitle(newtitle)
                    notify(ObjectModifiedEvent(obj))
                if newid and obid != newid:
                    parent = aq_parent(aq_inner(obj))
                    # Make sure newid is safe
                    newid = INameChooser(parent).chooseName(newid, obj)
                    # Update the default_page on the parent.
                    context_state = getMultiAdapter(
                        (obj, self.request), name='plone_context_state')
                    if context_state.is_default_page():
                        parent.setDefaultPage(newid)
                    parent.manage_renameObjects((obid, ), (newid, ))
                elif change_title:
                    # the rename will have already triggered a reindex
                    obj.reindexObject()
            except ConflictError:
                raise
            except Exception as e:
                sp.rollback()
                logger.error(u'Error renaming "{title}": "{exception}"'
                    .format(title=title.decode('utf8'), exception=e))
                self.errors.append(_(u'Error renaming ${title}', mapping={
                    'title': title.decode('utf8')}))

        return self.message(missing)


# override the layout so it only shows the folder contents
FC_MINIMAL_LAYOUT = """<!doctype html>
<html class="no-js" lang="en-us" data-gridsystem="bs3">
  <head>
    <meta charset="utf-8" />

    <link data-tile="${portal_url}/@@castle.cms.metadata"/>

    <link data-tile="${portal_url}/@@plone.app.standardtiles.stylesheets"/>

    <meta name="generator" content="Castle - https://www.wildcardcorp.com"/>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  </head>
  <body id="visual-portal-wrapper">
    <div data-tile="${context_url}/@@plone.app.standardtiles.toolbar" />

      <!-- Main Content -->
      <div class="row" id="main-content-container">
        <div id="main-content">
            ${structure: content.main}
        </div>
      </div>

      <link data-tile="${portal_url}/@@plone.app.standardtiles.javascripts"/>
  </body>
</html>
"""


class FolderContentsView(BaseFolderContentsView):

    def __call__(self):
        self.request.environ['X-CASTLE-LAYOUT'] = FC_MINIMAL_LAYOUT
        return super(FolderContentsView, self).__call__()

    def get_columns(self):
        columns = super(FolderContentsView, self).get_columns()
        sm = getSecurityManager()
        if sm.checkPermission(DEFAULT_PERMISSION_SECURE, self.context):
            # if has permission, add more more columns
            columns.update({
                'Creator': 'Creator',
                'last_modified_by': 'Last modified by'
            })
        return columns



from AccessControl.SecurityManagement import newSecurityManager
from castle.cms import audit
from castle.cms import utils
from castle.cms.utils import ESConnectionFactoryFactory
from collective.elasticsearch.es import ElasticSearchCatalog
from DateTime import DateTime
from plone import api
from plone.app.uuid.utils import uuidToObject
from plone.registry.interfaces import IRegistry
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component.hooks import setSite

import transaction


LAST_CHECKED_KEY = 'castle.forced-publish-last-checked'


EMAIL_BODY = """
<p>This content has been forced published:</p>
"""
EMAIL_BODY_ITEM = """
<li>
    <a href="{content_url}">{content_title}</a> - Published by {user_name}<br />
    <span class="font-size: 80%; color: #DDD">{comments}</span>
</li>"""


def check_site(site):
    # XXX will store when last check was so we always only look back
    # to previous check time
    setSite(site)
    catalog = api.portal.get_tool('portal_catalog')
    es = ElasticSearchCatalog(catalog)
    if not es.enabled:
        return

    index_name = audit.get_index_name()
    es = ESConnectionFactoryFactory()()

    sannotations = IAnnotations(site)
    last_checked = sannotations.get(LAST_CHECKED_KEY)
    if last_checked is None:
        last_checked = DateTime() - 30

    filters = [
        {'term': {'type': 'workflow'}},
        {'range': {'date': {'gt': last_checked.ISO8601()}}}
    ]

    if len(filters) > 1:
        qfilter = {'and': filters}
    else:
        qfilter = filters[0]
    query = {
        "query": {
            'filtered': {
                'filter': qfilter,
                'query': {'match_all': {}}
            }
        }
    }
    results = es.search(
        index=index_name,
        doc_type=audit.es_doc_type,
        body=query,
        sort='date:desc',
        size=1000)
    hits = results['hits']['hits']

    workflow = api.portal.get_tool('portal_workflow')
    forced = []
    checked = []
    for hit in hits:
        hit = hit['_source']
        if hit['object'] in checked:
            continue
        try:
            ob = uuidToObject(hit['object'])
            checked.append(hit['object'])
        except:
            continue

        try:
            review_history = workflow.getInfoFor(ob, 'review_history')
            if not review_history:
                continue

            for r in reversed(review_history):
                if (not r['action'] or r['review_state'] != 'published' or
                        not r.get('comments', '').startswith('OVERRIDE:')):
                    continue
                if r['time'] < last_checked:
                    # just quit now, we're getting to older history that we don't care about
                    break
                forced.append({
                    'ob': ob,
                    'history_entry': r
                })
        except WorkflowException:
            continue

    if len(forced) > 0:
        # sent out email to admins
        site_url = site.absolute_url()
        registry = getUtility(IRegistry)
        public_url = registry.get('plone.public_url')
        if not public_url:
            public_url = site_url
        email_html = EMAIL_BODY + '<ul>'
        for item in forced:
            ob = item['ob']
            wf_entry = item['history_entry']
            try:
                user = api.user.get(wf_entry['actor'])
                user_name = user.getProperty('fullname') or user.getId()
            except:
                user_name = wf_entry['actor']
            email_html += EMAIL_BODY_ITEM.format(
                content_url=ob.absolute_url().replace(site_url, public_url),
                content_title=ob.Title(),
                user_name=user_name,
                comments=wf_entry.get('comments', '')
            )
        email_html += '</ul>'
        email_subject = "Forced content publication update(Site: %s)" % (
            api.portal.get_registry_record('plone.site_title'))

        for user in api.user.get_users():
            user_roles = api.user.get_roles(user=user)
            email = user.getProperty('email')
            if (('Manager' not in user_roles and 'Site Administrator' not in user_roles) or
                    not email):
                continue

            utils.send_email(email, email_subject, html=email_html)

    site._p_jar.sync()
    sannotations[LAST_CHECKED_KEY] = DateTime()
    transaction.commit()


def run(app):
    singleton.SingleInstance('forcedpublishalert')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            check_site(obj)


if __name__ == '__main__':
    run(app)  # noqa



from Acquisition import aq_inner
from plone.app.contenttypes import browser
from Products.CMFCore.utils import getToolByName
from Products.MimetypesRegistry.MimeTypeItem import guess_icon_path
from zope.component import getMultiAdapter


class FileView(browser.file.FileView):

    @property
    def moved(self):
        return getattr(self.context.file, 'original_filename', None) is not None

    @property
    def filename(self):
        fi = self.context.file
        return getattr(fi, 'original_filename', fi.filename)

    @property
    def size(self):
        fi = self.context.file
        return getattr(fi, 'original_size', fi.getSize())

    @property
    def content_type(self):
        fi = self.context.file
        return getattr(fi, 'original_content_type', fi.contentType)

    def get_mimetype_icon(self):
        context = aq_inner(self.context)
        pstate = getMultiAdapter(
            (context, self.request),
            name=u'plone_portal_state'
        )
        portal_url = pstate.portal_url()
        mtr = getToolByName(context, "mimetypes_registry")
        mime = []
        if self.content_type:
            mime.append(mtr.lookup(self.content_type))
        if self.filename:
            mime.append(mtr.lookupExtension(self.filename))
        mime.append(mtr.lookup("application/octet-stream")[0])
        icon_paths = [m.icon_path for m in mime if hasattr(m, 'icon_path')]
        if icon_paths:
            return portal_url + "/" + icon_paths[0]

        return portal_url + "/" + guess_icon_path(mime[0])



from zope.interface import Interface


class IAuthenticator(Interface):

    def __init__(context, request):
        pass

    def change_password(member, new_password):
        pass

    def issue_country_exception(user, country):
        pass

    def login(user):
        pass

    def authenticate(username=None, password=None, country=None):
        pass

    def issue_2factor_code(username):
        pass

    def authorize_2factor(username, code):
        pass

    def get_supported_auth_schemes():
        pass



from castle.cms.utils import is_mosaic_edit_form
from plone.app.layout.viewlets import toolbar
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class ToolbarViewletManager(toolbar.ToolbarViewletManager):
    """
    so we can provide info on user tasks
    """

    custom_template = ViewPageTemplateFile('templates/toolbar.pt')

    @property
    def show(self):
        if (self.portal_state.anonymous() or
                self.request.form.get('VIEW_AS_ANONYMOUS_VIEW') == 'true'):
            return False

        if is_mosaic_edit_form(self.request):
            return False
        return True



# -*- coding: utf-8 -*-
from castle.cms.services import twitter
from castle.cms.interfaces import ISocialMediaSchema
from plone import api
from plone.app.registry.browser import controlpanel
from plone.protect.interfaces import IDisableCSRFProtection
from plone.registry.interfaces import IRegistry
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form.interfaces import HIDDEN_MODE
from zope.component import getUtility
from zope.interface import alsoProvides


class AuthorizeTwitter(BrowserView):

    def __call__(self):
        if 'oauth_verifier' in self.request.form:
            alsoProvides(self.request, IDisableCSRFProtection)
            tokens = twitter.authorize()
            registry = getUtility(IRegistry)
            registry['plone.twitter_oauth_token'] = tokens['oauth_token']
            registry['plone.twitter_oauth_secret'] = tokens['oauth_token_secret']
            registry['plone.twitter_screen_name'] = tokens['screen_name']
            self.request.response.redirect('%s/@@social-controlpanel' % (
                api.portal.get().absolute_url()))
        else:
            self.request.response.redirect(twitter.get_auth_url())


class SocialControlPanelForm(controlpanel.RegistryEditForm):

    id = "SocialControlPanel"
    label = _(u"Social Media Settings")
    description = _("Social media sharing settings.")
    schema = ISocialMediaSchema
    schema_prefix = "plone"

    def updateWidgets(self):
        super(SocialControlPanelForm, self).updateWidgets()
        self.widgets["twitter_oauth_token"].mode = HIDDEN_MODE
        self.widgets["twitter_oauth_secret"].mode = HIDDEN_MODE
        self.widgets["twitter_screen_name"].mode = HIDDEN_MODE


class SocialControlPanel(controlpanel.ControlPanelFormWrapper):
    form = SocialControlPanelForm

    index = ViewPageTemplateFile('templates/social.pt')

    def hasTwitterApp(self):
        keys = twitter.get_keys()
        if keys[0] is None or keys[1] is None:
            return False
        return True

    def hasTwitterUserAuth(self):
        return twitter.get_auth() is not None



# -*- coding: utf-8 -*-
from plone.app.contenttypes import _
from plone.app.textfield import RichText as RichTextField
from plone.autoform.interfaces import IFormFieldProvider
from plone.autoform.view import WidgetsView
from plone.autoform import directives as form
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider
from plone.app.z3cform.widget import RichTextFieldWidget


@provider(IFormFieldProvider)
class IOverview(model.Schema):

    overview = RichTextField(
        title=_(u'Overview'),
        description=u"Overview for what the content is about. This is longer than "
                    u"the description and allows for rich text",
        required=False,
    )
    form.widget('overview', RichTextFieldWidget)


@implementer(IOverview)
@adapter(IDexterityContent)
class Overview(object):

    def __init__(self, context):
        self.context = context


class WidgetView(WidgetsView):
    schema = IOverview



from castle.cms.tiles.base import BaseTile
from castle.cms.tiles.base import DisplayTypeTileMixin
from castle.cms.tiles.views import BaseTileView
from castle.cms.tiles.views import TileViewsSource
from castle.cms.utils import parse_query_from_data
from castle.cms.widgets import PreviewSelectFieldWidget
from castle.cms.widgets import QueryFieldWidget
from DateTime import DateTime
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from plone.autoform import directives as form
from plone.memoize.instance import memoize
from plone.supermodel import model
from plone.tiles.interfaces import IPersistentTile
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib import urlencode
from urlparse import parse_qsl
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.interface import implements
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import json


def _list(val):
    if type(val) not in (list, set, tuple):
        val = [val]
    return val


def _query_val(val):
    if isinstance(val, dict):
        if 'query' in val:
            val = val['query']
    return _list(val)


class DefaultView(BaseTileView):
    name = 'default'
    preview = '++plone++castle/images/previews/querylisting/default.png'
    order = 0
    index = ViewPageTemplateFile('templates/querylisting/default.pt')
    tile_name = 'querylisting'


class DefaultNaturalView(BaseTileView):
    name = 'default-natural'
    label = 'Default(Natural image)'
    preview = '++plone++castle/images/previews/querylisting/default.png'
    order = 0
    index = ViewPageTemplateFile('templates/querylisting/default-natural.pt')
    tile_name = 'querylisting'


class CompactView(BaseTileView):
    name = 'compact'
    preview = '++plone++castle/images/previews/querylisting/compact.png'
    order = 1
    index = ViewPageTemplateFile('templates/querylisting/compact.pt')
    tile_name = 'querylisting'


class GridView(BaseTileView):
    name = 'grid'
    preview = '++plone++castle/images/previews/querylisting/grid.png'
    order = 2
    index = ViewPageTemplateFile('templates/querylisting/grid.pt')
    tile_name = 'querylisting'


class TiledView(BaseTileView):
    name = 'tiled'
    preview = '++plone++castle/images/previews/querylisting/tiled.png'
    order = 3
    index = ViewPageTemplateFile('templates/querylisting/tiled.pt')
    tile_name = 'querylisting'


class TaggedView(BaseTileView):
    name = 'tagged'
    preview = '++plone++castle/images/previews/querylisting/tagged.png'
    order = 5
    index = ViewPageTemplateFile('templates/querylisting/tagged.pt')
    tile_name = 'querylisting'


class BlogView(BaseTileView):
    name = 'blog'
    order = 5
    index = ViewPageTemplateFile('templates/querylisting/blog.pt')
    tile_name = 'querylisting'


class QueryListingTile(BaseTile, DisplayTypeTileMixin):
    implements(IPersistentTile)

    display_type_name = 'querylisting'
    display_type_default = 'default'
    display_type_fallback_view = DefaultView

    query_attrs = ('SearchableText', 'Subject', 'sort_on', 'Title')
    mapped_tags = set([
        'Flyer',
        'Handbook',
        'Report',
        'Form'
    ])

    def get_tag(self, item):
        # mapping of potential assigned tags and content types
        # these need to be implemented by underlying theme...
        intersect = set(item.Subject) & self.mapped_tags
        if len(intersect) > 0:
            title = list(intersect)[0]
            search_param = '?Subject=' + title
            class_name = 'castle-tag-' + title.lower().replace(' ', '-')
        else:
            title = item.Type
            type_name = item.portal_type
            search_param = '?portal_type=' + type_name
            class_name = 'castle-tag-' + type_name.lower().replace(' ', '-')
        return {
            'url': '%s/@@search%s' % (
                self.site.absolute_url(),
                search_param),
            'className': class_name,
            'title': title
        }

    def get_query(self):
        parsed = parse_query_from_data(self.data, self.context)
        # XXX we're forcing location queries to be depth of 1
        if 'path' in parsed and 'depth' not in parsed['path']:
            parsed['path']['depth'] = 1
        if 'sort_on' not in parsed:
            parsed['sort_on'] = 'effective'  # defaults to this

        if 'selected-year' in self.request.form:
            # need to turn this into a date query
            year = self.request.form['selected-year']
            try:
                start = DateTime(abs(int(year)), 1, 1)
                end = DateTime(int(year) + 1, 1, 1) - 1
                parsed['effective'] = {
                    'query': (start, end),
                    'range': 'min:max'
                }
            except (KeyError, AttributeError, ValueError, TypeError):
                pass
        return parsed

    @property
    def display_fields(self):
        df = self.data.get('display_fields', None)
        if df is None:
            df = ('image', 'description')
        return df

    @property
    def limit(self):
        return self.data.get('limit', 20) or 20

    @memoize
    def results(self):
        catalog = getToolByName(self.context, 'portal_catalog')

        # there is a special case with Subject queries...
        # subject queries are OR, so if they are in the original query
        # and have a further filter with a different Subject, we need to
        # do some manual filtering. This can be potentially slow....
        # it's an edge case, so hopefully it's okay...
        query = self.get_query()
        form = self.get_form()
        subject_filter = None
        for attr in self.query_attrs:
            if form.get(attr):
                val = _list(form.get(attr))
                if attr in query and len(set(val) & set(_query_val(query[attr]))) > 0:
                    # matches here when subject is in the original query
                    # so we're trying to turn this into an AND query with manual
                    # filtering later
                    subject_filter = val[0]
                else:
                    if attr == 'Subject':
                        query[attr] = val
                    else:
                        query[attr] = val[0]

        result = catalog(**query)
        if subject_filter is not None:
            # special case where we have to further filter...
            result = [item for item in result
                      if item.Subject and subject_filter in item.Subject]
        try:
            form = self.get_form()
            page = int(form.get('page', 1)) - 1
        except:
            page = 0
        page = max(page, 0)
        start = page * self.limit
        end = start + self.limit
        return {
            'total': len(result),
            'page': page + 1,
            'items': result[start:end]
        }

    def _next_url(self, url, page):
        params = {}
        if '?' in url:
            url, _, params = url.partition('?')
            params = dict(parse_qsl(params))
        params['page'] = page + 1
        return url + '?' + urlencode(params)

    def next_url(self, page):
        return self._next_url(self.this_url, page)

    def get_form(self):
        try:
            return self.request.original_data
        except AttributeError:
            return self.request.form

    @property
    @memoize
    def this_url(self):
        url = '{}/@@{}/{}'.format(
            self.context.absolute_url(), self.__name__, self.id or ''
        )
        params = {}
        form = self.get_form()
        for attr in self.query_attrs:
            if form.get(attr):
                params[attr] = form.get(attr)
        if len(params) > 0:
            url += '?' + urlencode(params)
        return url

    @property
    def filter_pattern_config(self):
        config = {
            'tags': self.data.get('available_tags', []) or []
        }
        form = self.get_form()
        config['query'] = {}
        for attr in self.query_attrs:
            if form.get(attr):
                config['query'][attr] = form.get(attr)
        if ('Subject' in config['query'] and
                isinstance(config['query']['Subject'], basestring)):
            config['query']['Subject'] = [config['query']['Subject']]
        config['ajaxResults'] = {
            'url': self.this_url,
            'selector': '#query-results-%s' % self.id or ''
        }
        return json.dumps(config)


class IQueryListingTileSchema(model.Schema):

    title = schema.TextLine(
        title=u'Title',
        description=u'Provide title above listing',
        required=False,
        default=None
    )

    form.widget(query=QueryFieldWidget)
    query = schema.List(
        title=u'Base query',
        description=u"This query can be customized base on user selection",
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False
    )

    sort_on = schema.TextLine(
        title=u'Sort on',
        description=u"Sort on this index",
        required=False,
    )

    sort_reversed = schema.Bool(
        title=u'Reversed order',
        description=u'Sort the results in reversed order',
        required=False,
    )

    limit = schema.Int(
        title=u'Limit',
        description=u'Limit Search Results',
        required=False,
        default=15,
        min=1,
    )

    form.widget(
        'available_tags',
        AjaxSelectFieldWidget,
        vocabulary='plone.app.vocabularies.Keywords'
    )
    available_tags = schema.Tuple(
        title=u'Tags',
        description=u'Available tags to select from the query widget',
        value_type=schema.TextLine(),
        required=False,
        missing_value=()
    )

    form.widget('display_fields', CheckBoxFieldWidget)
    display_fields = schema.Tuple(
        title=u'Display fields',
        description=u'Fields that should show on the listing',
        default=(
            'image',
            'description'
        ),
        value_type=schema.Choice(
            vocabulary=SimpleVocabulary([
                SimpleTerm('image', 'image', u'Image'),
                SimpleTerm('date', 'date', u'Date'),
                SimpleTerm('description', 'description', u'Overview/Summary')
            ])
        )
    )

    form.widget('display_type', PreviewSelectFieldWidget,
                tile_name='querylisting')
    display_type = schema.Choice(
        title=u"Display Type",
        source=TileViewsSource('querylisting'),
        default='default'
    )



from castle.cms import ua
from castle.cms.lockout import get_active_sessions
from castle.cms.lockout import SessionManager
from plone import api
from Products.Five import BrowserView


class SessionsView(BrowserView):
    label = 'Active Sessions'

    def __call__(self):

        if self.request.REQUEST_METHOD == 'POST' and self.request.form.get('id'):
            session_id = self.request.form.get('id')
            user_id = self.request.form.get('userid')
            user = api.user.get(user_id)
            sm = SessionManager(self.context, self.request, user)
            sm.session_id = session_id
            sm.expire()

        self.sessions = get_active_sessions()

        return super(SessionsView, self).__call__()

    def parse_ua(self, session):
        return '%s: %s' % ua.simple_detect(session['ua'])



from castle.cms import cache
from castle.cms import social
from castle.cms.services import google
from plone import api
from Products.Five import BrowserView
from zope.component import getMultiAdapter

import json


class AnalyticsView(BrowserView):

    def __call__(self):
        self.request.response.setHeader('Content-type', 'application/json')
        paths = self.get_paths()

        if self.request.get('api') == 'ga':
            data = self.ga_api_call(paths)
        else:
            data = social.get_stats(self.context)
            if data:
                data = dict(data)

        return json.dumps({
            'paths': paths,
            'data': data
        })

    def ga_api_call(self, paths):
        service = google.get_ga_service()
        if not service:
            return

        profile = self.get_ga_profile(service)
        if not profile:
            return

        params = json.loads(self.request.get('params'))

        if self.request.get('type') == 'realtime':
            ga = service.data().realtime()
            if not params.pop('global', False):
                # need to restrict by filters
                path_query = ','.join(['rt:pagePath==%s' % p for p in paths])
                params['filters'] = path_query
        else:
            if not params.pop('global', False):
                # need to restrict by filters
                path_query = ','.join(['ga:pagePath==%s' % p for p in paths])
                params['filters'] = path_query
            ga = service.data().ga()

        cache_key = '-'.join(api.portal.get().getPhysicalPath()[1:])
        for key, value in params.items():
            cache_key += '%s=%s' % (key, value)

        try:
            result = cache.get(cache_key)
        except:
            result = None
        if result is None:
            query = ga.get(ids='ga:' + profile, **params)
            result = query.execute()
            cache_duration = self.request.get('cache_duration')
            if cache_duration:
                cache.set(cache_key, result, int(cache_duration))
        return result

    def get_ga_profile(self, service):
        cache_key = '%s-ga-profile' % '-'.join(api.portal.get().getPhysicalPath()[1:])
        try:
            profile = cache.get(cache_key)
        except:
            profile = None
        if profile is None:
            profile = google.get_ga_profile(service)
            cache.set(cache_key, profile, 60 * 60 * 1)
        return profile

    def get_paths(self):
        site_path = '/'.join(api.portal.get().getPhysicalPath())
        context_path = '/'.join(self.context.getPhysicalPath())
        base_path = context_path[len(site_path):]
        paths = [base_path, base_path + '/view']

        context_state = getMultiAdapter((self.context, self.request),
                                        name='plone_context_state')
        if context_state.is_portal_root():
            paths.append('/')
            paths.append('/main-page')
        return list(set(paths))



from castle.cms import subscribe
from castle.cms import texting
from castle.cms.utils import send_email
from castle.cms.utils import verify_recaptcha
from castle.cms.widgets import ReCaptchaFieldWidget
from plone import api
from plone.app.users.schema import checkEmailAddress
from plone.autoform.form import AutoExtensibleForm
from plone.protect.interfaces import IDisableCSRFProtection
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ISiteSchema
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib import urlencode
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form import interfaces
from z3c.form.action import ActionErrorOccurred
from z3c.form.interfaces import WidgetActionExecutionError
from zope import schema
from zope.component import getUtility
from zope.component import queryUtility
from zope.event import notify
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.interface import Invalid

import phonenumbers
import string


def check_phone_number(val):
    if not val:
        return
    number = ''.join([l for l in val if l in string.digits])

    if len(number) == 10:
        number = '1' + number

    number = '+' + number
    try:
        number = phonenumbers.parse(number, None)
    except:
        raise Invalid('Not a valid phone number')

    if not phonenumbers.is_valid_number(number):
        raise Invalid('Not a valid phone number')
    return True


class ISubscribeForm(Interface):
    name = schema.ASCIILine(
        title=u'Full name')

    email = schema.ASCIILine(
        title=u'E-mail',
        description=u'',
        required=True,
        constraint=checkEmailAddress)

    phone_number = schema.TextLine(
        title=u'Phone number',
        description=u'If you would like to receive text message alerts. ',
        required=False,
        constraint=check_phone_number)

    categories = schema.List(
        title=u'Categories',
        description=u"Narrow the types of content you'd like to recieve. "
                    u"Leave blank to recieve everything.",
        required=False,
        value_type=schema.Choice(
            vocabulary='castle.cms.vocabularies.EmailCategories'
        )
    )

    captcha = schema.TextLine(
        title=u"Captcha",
        required=False)


class BaseForm(AutoExtensibleForm, form.Form):

    def send_text_message(self, subscriber):
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema,
                                              prefix="plone",
                                              check=False)
        text_message = '%s phone verification code: %s' % (
            site_settings.site_title,
            subscriber['code'][-6:])
        return texting.send(text_message, subscriber['phone_number'])


class SubscribeForm(BaseForm):
    label = u"Subscribe"
    description = u"Subscribe to this site for notifications"
    formErrorsMessage = 'There were errors.'
    ignoreContext = True
    schema = ISubscribeForm
    template = ViewPageTemplateFile('templates/subscribe.pt')
    sent = False
    subscribed = False

    def __init__(self, context, request):
        super(SubscribeForm, self).__init__(context, request)
        registry = queryUtility(IRegistry)
        self.has_captcha = registry.get('castle.recaptcha_private_key') not in (None, '')
        self.has_texting = registry.get('castle.plivio_auth_id') not in (None, '')
        self.isAnon = None

    def send_mail(self, email, item):
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema,
                                              prefix="plone",
                                              check=False)
        url = '%s/@@subscribe?confirmed_email=%s&confirmed_code=%s' % (
            self.context.absolute_url(), email, item['code'])
        text = """
Copy and paste this url into your web browser to confirm your address: %s
""" % url
        html = """
<p>You have requested subscription, please
<a href="%s">confirm your email address by clicking on this link</a>.
</p>
<p>
If that does not work, copy and paste this urls into your web browser: %s
</p>""" % (url, url)
        send_email(
            [email], "Email Confirmation for subscription(%s)" % site_settings.site_title,
            html=html, text=text)

    def updateFields(self):
        super(SubscribeForm, self).updateFields()
        if self.isAnon is None:
            portal_membership = getToolByName(self.context, 'portal_membership')
            self.isAnon = portal_membership.isAnonymousUser()

        if self.has_captcha and self.isAnon:
            self.fields['captcha'].widgetFactory = ReCaptchaFieldWidget
        else:
            self.fields['captcha'].mode = interfaces.HIDDEN_MODE
        if not self.has_texting:
            self.fields['phone_number'].mode = interfaces.HIDDEN_MODE

    @button.buttonAndHandler(u'Subscribe', name='subscribe')
    def action_subscribe(self, action):
        data, errors = self.extractData()

        if self.has_captcha and self.isAnon:
            if not verify_recaptcha(self.request):
                notify(
                    ActionErrorOccurred(
                        action,
                        WidgetActionExecutionError('captcha', Invalid('Invalid Recaptcha'))))
                return

        subsciber = subscribe.get_subscriber(data.get('email'))
        if subsciber:
            notify(
                ActionErrorOccurred(
                    action,
                    WidgetActionExecutionError('email', Invalid('User already subscribed'))))
            return

        if not errors:
            item = subscribe.register(data['email'], data)
            self.send_mail(data['email'], item)
            self.sent = True
            api.portal.show_message(
                'Verification email has been sent to your email', request=self.request, type='info')
            if self.has_texting and data.get('phone_number'):
                if not self.send_text_message(item):
                    api.portal.show_message('Error sending code', request=self.request,
                                            type='error')
                else:
                    api.portal.show_message('Code texted to your number to verify',
                                            request=self.request, type='info')
                self.request.response.redirect('%s/@@subscribe-phone?%s' % (
                    self.context.absolute_url(),
                    urlencode({
                        'form.widgets.email': item['email'],
                        'form.widgets.phone_number': item.get('phone_number', '')
                        })
                    ))

    def __call__(self):
        registry = queryUtility(IRegistry)
        self.subscriptions_enabled = registry.get(
            'plone.enable_notification_subscriptions', False)
        portal_membership = getToolByName(self.context, 'portal_membership')
        self.isAnon = portal_membership.isAnonymousUser()

        if not self.has_captcha or not self.subscriptions_enabled:
            api.portal.show_message(
                'Subscriptions are not enabled on this site', request=self.request, type='error')
            return self.request.response.redirect(self.context.absolute_url())
        if 'confirmed_code' in self.request.form and 'confirmed_email' in self.request.form:
            try:
                alsoProvides(self.request, IDisableCSRFProtection)
                subscribe.confirm(self.request.form['confirmed_email'],
                                  self.request.form['confirmed_code'])
                api.portal.show_message('Succussfully subscribed to portal', request=self.request)
                self.subscribed = True
            except subscribe.InvalidEmailException:
                api.portal.show_message('Invalid Email', request=self.request, type='error')
            except subscribe.InvalidCodeException:
                api.portal.show_message('Invalid Code', request=self.request, type='error')

        return super(SubscribeForm, self).__call__()


class IConfirmPhoneForm(Interface):

    email = schema.ASCIILine(
        title=u'E-mail',
        description=u'',
        required=True,
        constraint=checkEmailAddress)

    phone_number = schema.TextLine(
        title=u'Phone number',
        description=u'Change number and click re-send to use a different number',
        constraint=check_phone_number)

    phone_number_code = schema.TextLine(
        title=u'Code',
        description=u'Code that was texted to your phone',
        required=False)


class ConfirmPhoneForm(BaseForm):
    label = u"Confirm phone number"
    description = u""
    schema = IConfirmPhoneForm
    template = ViewPageTemplateFile('templates/subscribe-phone.pt')
    confirmed = False
    formErrorsMessage = 'There were errors.'
    ignoreContext = True

    def updateFields(self):
        super(ConfirmPhoneForm, self).updateFields()
        self.fields['email'].mode = interfaces.HIDDEN_MODE

    @button.buttonAndHandler(u'Confirm', name='subscribe')
    def action_subscribe(self, action):
        data, errors = self.extractData()

        subscriber = subscribe.get_subscriber(data['email'])
        if errors:
            return

        if subscriber.get('phone_number') != data['phone_number']:
            subscriber['phone_number'] = data['phone_number']
            if not self.send_text_message(subscriber):
                api.portal.show_message('Error sending code', request=self.request, type='error')
            else:
                api.portal.show_message('Phone number changed, code re-sent',
                                        request=self.request, type='info')

        if not data.get('phone_number_code'):
            ActionErrorOccurred(
                action,
                WidgetActionExecutionError('phone_number_code', Invalid('No code specified')))

        if not errors:
            try:
                alsoProvides(self.request, IDisableCSRFProtection)
                subscribe.confirm_phone_number(data['email'], data['phone_number_code'])
                self.confirmed = True
                api.portal.show_message(
                    'Phone number successfully confirmed', request=self.request, type='info')
            except subscribe.InvalidEmailException:
                api.portal.show_message('Invalid Email', request=self.request, type='error')
            except subscribe.InvalidCodeException:
                api.portal.show_message('Invalid Code', request=self.request, type='error')

    @button.buttonAndHandler(u'Re-send', name='resend')
    def action_resend(self, action):
        data, errors = self.extractData()

        subscriber = subscribe.get_subscriber(data['email'])
        if errors:
            return

        if subscriber.get('phone_number') != data['phone_number']:
            subscriber['phone_number'] = data['phone_number']

        if not self.send_text_message(subscriber):
            api.portal.show_message('Error sending code', request=self.request, type='error')
        else:
            api.portal.show_message('Code sent', request=self.request, type='info')

class ChangeSubscription(form.EditForm):
    schema = ISubscribeForm
    fields = field.Fields(ISubscribeForm)
    label = u"Edit you subscription settings"
    ignoreContext = True
    validUser = True
    user = None
    email = None
    code = None

    def __init__(self, context, request):
        super(ChangeSubscription, self).__init__(context, request)

        if 'code' in request.form and 'email' in request.form:
            self.email = self.request.form['email']
            self.code = self.request.form['code']

            self.setUser()
            self.validateUser()
        else:
            api.portal.show_message('Invalid URL', request=self.request, type='error')
            self.validUser = False
            return

    def action(self):
        url = self.context.absolute_url() + '/@@changesubscription?'
        url += urlencode({
            'code': self.user['code'],
            'email': self.user['email']
        })
        return url

    @button.buttonAndHandler(u'Apply', name='apply')
    def action_apply(self, action):
        prefix = 'form.widgets.'
        user = subscribe.get(self.request.form[prefix + 'email'])
        form = self.request.form

        changed_items = ['phone_number', 'name', 'categories']

        for item in changed_items:
            if prefix + item in form:
                user[item] = form[prefix + item]
                self.widgets[item].value = form[prefix + item]
            else:
                # The categories widget isn't included if it's empty
                user[item] = []
                self.widgets[item].value = []

        self.widgets['categories'].update()

        api.portal.show_message('Subscription settings changed', request=self.request, type='info')

    def getContent(self):
        if not self.validUser:
            return

        return self.user

    def setUser(self):
        self.user = subscribe.get(self.email)

    def validateUser(self):
        if self.user['code'] != self.code:
            api.portal.show_message('Invalid URL', request=self.request, type='error')
            self.validUser = False

    def updateWidgets(self):
        super(ChangeSubscription, self).updateWidgets()
        if not self.validUser:
            return

        form_items = ['email', 'name', 'phone_number', 'categories']
        for item in form_items:
            self.widgets[item].value = self.user[item]

        self.widgets['categories'].update()

        self.widgets['email'].mode = interfaces.HIDDEN_MODE
        self.widgets['captcha'].mode = interfaces.HIDDEN_MODE

class Unsubscribe(BrowserView):
    unsubscribed = False
    error = None

    def __call__(self):
        form = self.request.form
        if 'code' in form and 'email' in form:
            email = form['email']
            subscriber = subscribe.get(email)
            if subscriber:
                if subscriber.get('code') == form['code']:
                    alsoProvides(self.request, IDisableCSRFProtection)
                    subscribe.remove(email)
                    self.unsubscribed = True
                else:
                    self.error = 'Invalid unsubscribe url'
        return super(Unsubscribe, self).__call__()



from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.dexterity.behaviors.discussion import IAllowDiscussion
from plone.app.discussion.browser.conversation import ConversationView
from plone.app.discussion.interfaces import IDiscussionSettings
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.component import queryUtility


IAllowDiscussion['allow_discussion'].description = (u'Allow discussion for this content '
                                                    u'object. If set on folder, it will '
                                                    u'enable for all items inside folder.')


class CastleConversationView(ConversationView):

    def enabled(self):
        """
        We only support dexterity anyways so do complete check here...

        Returns True if discussion is enabled for this conversation.

        This method checks five different settings in order to figure out if
        discussion is enable on a specific content object:

        1) Check if discussion is enabled globally in the plone.app.discussion
           registry/control panel.

        2) Check if the allow_discussion boolean flag on the content object is
           set. If it is set to True or False, return the value. If it set to
           None, try further.

        3) Check if discussion is allowed for the content type.
        """
        context = aq_inner(self.context)

        # Fetch discussion registry
        registry = queryUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)

        # Check if discussion is allowed globally
        if not settings.globally_enabled:
            return False

        # Check if discussion is allowed on the content object
        if hasattr(context, "allow_discussion"):
            if context.allow_discussion is not None:
                return context.allow_discussion

        # now, check parent for setting...
        parent = aq_parent(context)
        if hasattr(parent, "allow_discussion"):
            if parent.allow_discussion:
                return True

        # Check if discussion is allowed on the content type
        portal_types = getToolByName(self, 'portal_types')
        document_fti = getattr(portal_types, context.portal_type)
        return document_fti.getProperty('allow_discussion')



# -*- coding: utf-8 -*-
import unittest

from castle.cms import trash
from castle.cms.interfaces import ITrashed
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles


class TestTrash(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_trash_object(self):
        doc = api.content.create(type='Document', id='doc1', container=self.portal)
        trash.object(doc)
        self.assertTrue(ITrashed.providedBy(doc))

    def test_restore_object(self):
        doc = api.content.create(type='Document', id='doc1', container=self.portal)
        trash.object(doc)
        self.assertTrue(ITrashed.providedBy(doc))
        trash.restore(doc)
        self.assertFalse(ITrashed.providedBy(doc))



from Acquisition import aq_inner
from castle.cms import texting
from castle.cms.browser.utils import Utils
from castle.cms.constants import ALL_SUBSCRIBERS
from castle.cms.interfaces import IAnnoucementData
from castle.cms.tasks import send_email
from castle.cms.tasks import send_email_to_subscribers
from castle.cms.widgets import AjaxSelectFieldWidget
from castle.cms.widgets import SelectFieldWidget
from castle.cms.widgets import TinyMCETextFieldWidget
from plone import api
from plone.app.registry.browser import controlpanel
from plone.app.textfield import RichText
from plone.app.textfield.value import RichTextValue
from plone.autoform import directives
from plone.autoform.form import AutoExtensibleForm
from plone.outputfilters import apply_filters
from plone.outputfilters.interfaces import IFilter
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form import form
from zope import schema
from zope.component import getAdapters
from zope.component import getUtility


class AnnouncementsControlPanelForm(controlpanel.RegistryEditForm):
    schema = IAnnoucementData
    schema_prefix = 'castle'
    id = "AnnouncementsControlPanel"
    label = u"Announcements"
    description = ""

    def updateFields(self):
        super(AnnouncementsControlPanelForm, self).updateFields()
        self.fields['site_announcement'].widgetFactory = TinyMCETextFieldWidget


class ISendEmailUsersForm(model.Schema):
    subject = schema.ASCIILine(
        title=u'Subject')

    directives.widget(
        'send_to_groups',
        AjaxSelectFieldWidget,
        vocabulary='plone.app.vocabularies.Groups'
    )
    send_to_groups = schema.List(
        title=u'Send to groups',
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Groups'
        )
    )

    directives.widget(
        'send_to_users',
        AjaxSelectFieldWidget,
        vocabulary='plone.app.vocabularies.Users'
    )
    send_to_users = schema.List(
        title=u'Send to users',
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Users'
        ))

    send_to_custom = schema.List(
        title=u'To(additional)',
        description=u'Additional email addresses, one per line, to '
                    u'send emails to.',
        value_type=schema.TextLine(),
        required=False
    )

    body = RichText(
        title=u'Body',
        description=u'Message body',
        default_mime_type='text/html',
        output_mime_type='text/html',
        allowed_mime_types=('text/html',),
        required=True)


class SendEmailUsersForm(AutoExtensibleForm, form.Form):
    schema = ISendEmailUsersForm

    ignoreContext = True

    @button.buttonAndHandler(u'Send', name='send')
    def handle_send(self, action):
        data, errors = self.extractData()
        if not errors:
            addresses = data['send_to_custom'] or []
            for group in data['send_to_groups'] or []:
                for user in api.user.get_users(groupname=group):
                    email = user.getProperty('email')
                    if email:
                        addresses.append(email)
            for username in data['send_to_users'] or []:
                user = api.user.get(username=username)
                if user:
                    email = user.getProperty('email')
                    if email:
                        addresses.append(email)

            utils = Utils(self.context, self.request)
            public_url = utils.get_public_url()
            html = data['body'].output

            filters = [f for _, f in getAdapters((self.context, self.request), IFilter)]
            html = apply_filters(filters, html)
            html = html.replace(self.context.absolute_url(), public_url.encode('utf8'))

            send_email.delay(list(set(addresses)), data['subject'], html=html)
            self.request.response.redirect('%s/@@announcements-controlpanel' % (
                self.context.absolute_url()))


class ISendEmailSubscribersForm(model.Schema):
    subject = schema.ASCIILine(
        title=u'Subject')

    directives.widget('send_to_categories', SelectFieldWidget)
    send_to_categories = schema.List(
        title=u'Send to categories',
        required=False,
        value_type=schema.Choice(
            vocabulary='castle.cms.vocabularies.EmailCategories'
        )
    )
    body = RichText(
        title=u'Body',
        description=u'Message body',
        default_mime_type='text/html',
        output_mime_type='text/html',
        allowed_mime_types=('text/html',),
        required=True,
        default=RichTextValue(
            u'<p></p><p></p>'
            u'<p><a href="{{change_url}}">Change your subscription settings</a></p>'
            u'<p><a href="{{unsubscribe_url}}">Unsubscribe from these messages</a></p>',
            'text/html', 'text/html'))


class SendEmailSubscribersForm(AutoExtensibleForm, form.Form):
    schema = ISendEmailSubscribersForm

    ignoreContext = True

    @button.buttonAndHandler(u'Send', name='send2')
    def handle_send2(self, action):
        data, errors = self.extractData()
        if not errors:
            utils = Utils(self.context, self.request)
            public_url = utils.get_public_url()
            html = data['body'].output

            filters = [f for _, f in getAdapters((self.context, self.request), IFilter)]
            html = apply_filters(filters, html)
            html = html.replace(self.context.absolute_url(), public_url.encode('utf8'))

            categories = set()
            if 'form.widgets.send_to_categories' in self.request.form:
                categories = set(self.request.form['form.widgets.send_to_categories'])

            send_email_to_subscribers.delay(data['subject'], html=html, categories=categories)

            api.portal.show_message(
                'Sending emails', request=self.request, type='info')
            self.request.response.redirect('%s/@@announcements-controlpanel' % (
                self.context.absolute_url()))


class ISendTextForm(model.Schema):

    text = schema.Text(
        title=u'Text message')


class SendTextForm(AutoExtensibleForm, form.Form):

    schema = ISendTextForm

    ignoreContext = True

    @button.buttonAndHandler(u'Send', name='text')
    def handle_text(self, action):
        data, errors = self.extractData()
        if not errors:
            texting.send(data['text'], ALL_SUBSCRIBERS)
            api.portal.show_message(
                'Text message sent', request=self.request, type='info')
            self.request.response.redirect('%s/@@announcements-controlpanel' % (
                self.context.absolute_url()))


class AnnouncementsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = AnnouncementsControlPanelForm
    index = ViewPageTemplateFile('templates/announcements.pt')

    hasTexting = False

    def __init__(self, context, request):
        super(AnnouncementsControlPanel, self).__init__(context, request)
        self.email_form = SendEmailUsersForm(aq_inner(context), request)
        self.email_subscribers_form = SendEmailSubscribersForm(aq_inner(context), request)
        self.text_subscribers_form = SendTextForm(aq_inner(context), request)

    def __call__(self):
        registry = getUtility(IRegistry)
        if (registry.get('castle.plivo_auth_id') and
                registry.get('castle.plivo_auth_token') and
                registry.get('castle.plivo_phone_number')):
            self.hasTexting = True
        self.email_form.update()
        self.email_subscribers_form.update()
        self.text_subscribers_form.update()
        return super(AnnouncementsControlPanel, self).__call__()



from castle.cms import audit
from castle.cms.utils import ESConnectionFactoryFactory
from cStringIO import StringIO
from plone import api
from plone.app.uuid.utils import uuidToObject
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility

import csv


class AuditView(BrowserView):
    label = 'My Activity Log'
    limit = 50
    user = True

    inner_template = ViewPageTemplateFile('templates/audit-inner.pt')

    def __call__(self):
        self._user_cache = {}
        self.site_path = '/'.join(self.context.getPhysicalPath())
        results = self.do_query()
        self.results = results['hits']['hits']
        self.total = results['hits']['total']

        if 'Export' in self.request.form.get('export', ''):
            return self.export()
        return super(AuditView, self).__call__()

    def render_inner(self):
        return self.inner_template()

    def get_user(self, userid):
        if not userid:
            return
        if userid in self._user_cache:
            return self._user_cache[userid]
        user = api.user.get(userid)
        if user is not None:
            self._user_cache[userid] = user
        return user

    @property
    def el_connected(self):
        registry = getUtility(IRegistry)
        if not registry.get('collective.elasticsearch.interfaces.IElasticSettings.enabled', False):
            return False
        es = ESConnectionFactoryFactory()()
        try:
            return es.cluster.health()['status'] in ('green', 'yellow')
        except:
            return False

    def get_obj(self, uid):
        return uuidToObject(uid)

    def get_path(self, data):
        path = data.get('path')
        if not path:
            return ''
        if path.startswith(self.site_path):
            return path[len(self.site_path):]
        return path

    def get_query(self):
        filters = []
        form = self.request.form
        if form.get('type'):
            filters.append(
                {'term': {'type': form.get('type')}}
            )
        if self.user:
            filters.append(
                {'term': {'user': api.user.get_current().getId()}}
            )
        else:
            if form.get('user'):
                filters.append(
                    {'term': {'user': form.get('user')}}
                )
        if form.get('content'):
            items = form.get('content').split(';')
            cqueries = []
            for item in items:
                cqueries.append({'term': {'object': item}})
            filters.append(
                {'or': cqueries}
            )
        if form.get('after'):
            filters.append(
                {'range': {'date': {'gte': form.get('after')}}}
            )
        if form.get('before'):
            filters.append(
                {'range': {'date': {'lte': form.get('before')}}}
            )
        if len(filters) == 0:
            query = {
                "query": {'match_all': {}}
            }
        else:
            if len(filters) > 1:
                qfilter = {'and': filters}
            else:
                qfilter = filters[0]
            query = {
                "query": {
                    'filtered': {
                        'filter': qfilter,
                        'query': {'match_all': {}}
                    }
                }
            }
        return query

    def export(self):
        index_name = audit.get_index_name()
        es = ESConnectionFactoryFactory()()
        query = self.get_query()
        results = es.search(
            index=index_name,
            doc_type=audit.es_doc_type,
            body=query,
            sort='date:desc',
            size=3000)
        output = StringIO()
        writer = csv.writer(output)

        writer.writerow(['Action', 'Path', 'User', 'Summary', 'Date'])
        for result in results['hits']['hits']:
            data = result['_source']
            writer.writerow([
                data['name'],
                self.get_path(data),
                data['user'],
                data['summary'],
                data['date']
            ])

        resp = self.request.response
        resp.setHeader('Content-Disposition', 'attachment; filename=export.csv')
        resp.setHeader('Content-Type', 'text/csv')
        output.seek(0)
        return output.read()

    def do_query(self):
        index_name = audit.get_index_name()
        es = ESConnectionFactoryFactory()()
        query = self.get_query()

        try:
            page = int(self.request.get('page', 1))
        except:
            page = 1
        start = (page - 1) * self.limit
        results = es.search(
            index=index_name,
            doc_type=audit.es_doc_type,
            body=query,
            sort='date:desc',
            from_=start,
            size=self.limit)

        return results



from time import time

from BTrees.OOBTree import OOBTree
from castle.cms.utils import make_random_key


class RegistrationStorage(object):

    attr_name = '_registration_confirmations'

    def __init__(self, context):
        self.context = context
        try:
            self._data = getattr(context, self.attr_name)
        except AttributeError:
            self._data = OOBTree()
            setattr(context, self.attr_name, self._data)

    def add(self, email, data=None):
        self.clean()
        email = email.lower()
        if data is None:
            data = {}
        data.update({
            'created': time(),
            'code': make_random_key(100)
        })
        self._data[email] = data
        return data

    def remove(self, email):
        if email.lower() in self._data:
            del self._data[email.lower()]

    def get(self, email):
        return self._data.get(email.lower())

    def clean(self):
        now = time()
        delete = []
        for email, item in self._data.items():
            if not item:
                delete.append(email)
                continue
            created = item['created']
            # delete all older than 1 hour
            if int((now - created) / 60 / 60) > 1:
                delete.append(email)
        for code in delete:
            del self._data[code]


class RegistrationReviewStorage(RegistrationStorage):
    attr_name = '_registrations_under_review'


from Products.CMFCore.FSFile import FSFile
from plone.resource.directory import FilesystemResourceDirectory
from Products.Five.browser.resource import DirectoryResource
from Products.Five.browser.resource import FileResource
from plone.resource.file import FilesystemFile
import os


def get_js_resource_object(site, resource):
    if resource.js:
        return site.unrestrictedTraverse(str(resource.js), None)


def get_resource_object(site, path):
    if path:
        return site.unrestrictedTraverse(str(path), None)


def get_url_resource_object(site, resource):
    if resource.url:
        return site.unrestrictedTraverse(resource.url, None)


def remove_extension(filepath):
    parts = filepath.split('.')
    return '.'.join(parts[:-1])


def recursive_resources(base, fs_path):
    paths = {}
    for filename in os.listdir(fs_path):
        filepath = os.path.join(fs_path, filename)
        path_name = os.path.join(base, filename)
        if os.path.isdir(filepath):
            paths.update(recursive_resources(path_name, filepath))
        elif filename.split('.')[-1] in ('js',):
            paths[path_name] = remove_extension(filepath)
        else:
            paths[path_name] = filepath
    return paths


def get_module_dir(mod):
    return os.path.dirname(mod.__file__)


def resource_to_path(resource, file_type='.js'):
    if isinstance(resource, FilesystemFile):
        return resource.path
    elif isinstance(resource, FileResource):
        return resource.chooseContext().path
    elif isinstance(resource, DirectoryResource):
        return resource.context.path
    elif isinstance(resource, FilesystemResourceDirectory):
        return resource.directory
    elif isinstance(resource, FSFile):
        return resource._filepath
    else:
        print("Missing resource type")
        return None



from castle.cms.interfaces import ISiteSchema
from plone.app.registry.browser import controlpanel
from plone.formwidget.namedfile.widget import NamedFileFieldWidget
from Products.CMFPlone.controlpanel.browser import site
from z3c.form.browser.checkbox import CheckBoxFieldWidget


class SiteControlPanelForm(site.SiteControlPanelForm):
    schema = ISiteSchema

    def updateFields(self):
        super(SiteControlPanelForm, self).updateFields()
        self.fields['roles_allowed_to_add_keywords'].widgetFactory = CheckBoxFieldWidget
        self.fields['site_icon'].widgetFactory = NamedFileFieldWidget


class SiteControlPanel(controlpanel.ControlPanelFormWrapper):
    form = SiteControlPanelForm



from castle.cms.tiles.content import ContentTile
from castle.cms.widgets import VideoRelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.component import getMultiAdapter
from zope.component.hooks import getSite
from zope.globalrequest import getRequest
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class VideoTile(ContentTile):
    default_display_fields = ()

    def render(self):
        return self.index()

    def get_video(self):
        video = self.data.get('video')
        if not video:
            return
        return self.utils.get_object(self.data['video'][0])

    @property
    def show_controls(self):
        return self.data.get('show_controls', True)

    @property
    def autoplay(self):
        return self.data.get('autoplay', False)

    @property
    def loop(self):
        return self.data.get('loop', False)

    def tweak_youtube_url(self, url):
        # add params for properties...
        url = self.utils.clean_youtube_url(url)
        url += '?'
        if self.autoplay:
            url += 'autoplay=1&'
        if not self.show_controls:
            url += 'controls=0&'
        if self.loop:
            url += 'loop=1'
        return url


class IVideoTileSchema(model.Schema):

    form.widget(video=VideoRelatedItemsFieldWidget)
    video = schema.List(
        title=u"Video file",
        description=u"Reference video file on the site.",
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    youtube_url = schema.TextLine(
        title=u'YouTube url',
        description=u'Or provide a YouTube url',
        required=False)

    @invariant
    def validate_video(data):
        if data.video and len(data.video) != 1:
            raise Invalid("Must select 1 video")
        if data.video and data.youtube_url:
            raise Invalid("You can not select both a video and a youtube url")
        if not data.video and not data.youtube_url:
            raise Invalid("Must select a video or a youtube url")
        if data.video:
            utils = getMultiAdapter((getSite(), getRequest()),
                                    name="castle-utils")
            obj = utils.get_object(data.video[0])
            if obj.portal_type != 'Video':
                raise Invalid('Must provide video file')

    form.widget('display_fields', CheckBoxFieldWidget)
    display_fields = schema.Tuple(
        title=u'Display fields',
        description=u'Fields that should show from the content',
        default=(),
        value_type=schema.Choice(
            vocabulary=SimpleVocabulary([
                SimpleTerm('title', 'title', u'Title'),
                SimpleTerm('description', 'description', u'Overview/Summary'),
                SimpleTerm('date', 'date', u'Date'),
                SimpleTerm('transcript', 'transcript', u'Transcript'),
            ])
        )
    )

    display_type = schema.Choice(
        title=u'Display type',
        required=True,
        default=u'landscape',
        vocabulary=SimpleVocabulary([
            SimpleTerm('landscape', 'landscape', u'Landscape'),
            SimpleTerm('square', 'square', u'Square'),
            SimpleTerm('fullwidth', 'fullwidth', u'Full width'),
        ])
    )

    show_controls = schema.Bool(
        title=u'Show controls',
        default=True
    )

    autoplay = schema.Bool(
        title=u'Autoplay',
        default=False
    )

    loop = schema.Bool(
        title=u'Loop',
        default=False
    )



from castle.cms.behaviors.location import ILocation
from castle.cms.interfaces import ILDData
from castle.cms.utils import site_has_icon
from plone.app.layout.globals.interfaces import IViewView
from plone.app.layout.navigation.defaultpage import getDefaultPage
from plone.tiles import Tile
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFPlone.log import logger
from unidecode import unidecode
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewlet
from zope.viewlet.interfaces import IViewletManager

import json


head_viewlets = {
    'plone.htmlhead.title': 'plone.htmlhead',
    'plone.links.author': 'plone.htmlhead.links',
    'plone.links.RSS': 'plone.htmlhead.links',
    'plone.links.canonical_url': 'plone.htmlhead.links',
    'plone.htmlhead.dublincore': 'plone.htmlhead',
    'plone.htmlhead.socialtags': 'plone.htmlhead'
}


def _date(obj, date):
    try:
        return getattr(obj, date)().ISO8601()
    except:
        return ''


class MetaDataTile(Tile):

    def get_navigation_links(self):
        return u"""
<link rel="home"
      title="Front page"
      href="{url}"/>
<link rel="contents"
      href="{url}/sitemap"/>""".format(url=self.root_url)

    def get_icons(self):
        if not site_has_icon():
            return ''
        return '''
<link rel="apple-touch-icon" sizes="180x180" href="{url}/site-icon.png">
<link rel="icon" type="image/png" href="{url}/site-icon.png?scale=32" sizes="32x32">
<link rel="icon" type="image/png" href="{url}/site-icon.png?scale=16" sizes="16x16">
<link rel="manifest" href="{url}/manifest.json">
<meta name="theme-color" content="#ffffff">'''.format(url=self.root_url)

    def _wrap_ld(self, data):
        return '<script type="application/ld+json">' + json.dumps(data) + '</script>'

    def get_ld_data(self):
        result = ''
        ld = ILDData(self.context, None)
        if ld is None:
            return ''
        result += self._wrap_ld(ld.get_data())

        if ISiteRoot.providedBy(self.context):
            try:
                page = self.context[getDefaultPage(self.context)]
                result += self._wrap_ld(ILDData(page).get_data())
            except AttributeError:
                pass

        return result

    def get_basic_tags(self):
        try:
            context = self.context
            if ISiteRoot.providedBy(context):
                try:
                    context = context[getDefaultPage(context)]
                except AttributeError:
                    pass
            try:
                subject = context.Subject()
            except:
                subject = []

            tags = {
                'description': context.Description(),
                'keywords': ','.join(subject),
                'modificationDate': _date(context, 'modified'),
                'publicationDate': _date(context, 'effective'),
                'expirationDate': _date(context, 'expires'),
                'generator': 'Castle CMS 1.0'
            }
            ldata = ILocation(context, None)
            if ldata is not None:
                if ldata.locations:
                    location = ldata.locations
                    if type(location) in (list, tuple, set):
                        location = location[0]
                    tags['location'] = location

            return ''.join([u'<meta name="{}" content="{}">'.format(name, value)
                            for name, value in tags.items()])
        except:
            return u''

    def get_search_link(self):
        return '''
<link rel="search"
      title="Search this site"
      href="{url}/@@search" />'''.format(
            url=self.root_url
          )

    def __call__(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        self.root_url = portal_state.navigation_root_url()

        result = u''
        alsoProvides(self, IViewView)
        for name, manager_name in head_viewlets.items():
            manager = queryMultiAdapter(
                (self.context, self.request, self),
                IViewletManager, name=manager_name
            )
            viewlet = queryMultiAdapter(
                (self.context, self.request, self, manager),
                IViewlet, name=name
            )
            if viewlet is not None:
                try:
                    viewlet.update()
                    result += viewlet.render()
                except:
                    logger.warn('Error rendering head viewlet %s' % name, exc_info=True)
        result += unidecode(self.get_basic_tags())
        result += unidecode(self.get_navigation_links())
        result += unidecode(self.get_ld_data())
        result += unidecode(self.get_icons())
        result += unidecode(self.get_search_link())
        return u'<html><head>%s</head></html>' % result
